
from .maica_utils import (
    MsgType,
    CommonMaicaException,
    CommonMaicaError,
    CommonMaicaWarning,
    CriticalMaicaError,
    MaicaPermissionError,
    MaicaInputError,
    MaicaResponseError,
    MaicaDbError,
    MaicaPermissionWarning,
    MaicaInputWarning,
    MaicaConnectionWarning,
    MaicaInternetWarning,
    FscPlain,
    AsyncCreator,
    LimitedList,
    LoginResult,
    ReUtils,
    default,
    wrap_ws_formatter,
    ellipsis_str,
    fuzzy_match,
    words_upper,
    sleep_forever,
    alt_tools,
    maica_assert,
    has_valid_content,
    proceed_agent_response,
    sync_messenger,
    messenger,
    load_env,
    wrap_run_in_exc,
    limit_length,
    get_json,
    get_host,
    vali_url,
    vali_date,
    is_today,
    strip_date,
    refill_date,
    add_seq_suffix,
    clean_text,
    try_load_json,
    hash_sha256,
    Decos,
)
from .connection_utils import DbPoolCoroutine, SqliteDbPoolCoroutine, ConnUtils, AiConnCoroutine, validate_input
from .setting_utils import MaicaSettings
from .account_utils import AccountCursor, encrypt_token, sign_message, verify_message, sort_message
from .container_utils import FullSocketsContainer
from .sb_utils import SideBoundCoroutine, SideFunctionCoroutine
from .get_a_sentence import SentenceOfTheDay
from .locater import locater, get_inner_path, get_outer_path

__all__ = [
    'MsgType',
    'CommonMaicaException',
    'CommonMaicaError',
    'CommonMaicaWarning',
    'CriticalMaicaError',
    'MaicaPermissionError',
    'MaicaInputError',
    'MaicaResponseError',
    'MaicaDbError',
    'MaicaPermissionWarning',
    'MaicaInputWarning',
    'MaicaConnectionWarning',
    'MaicaInternetWarning',
    'FscPlain',
    'AsyncCreator',
    'LimitedList',
    'LoginResult',
    'FullSocketsContainer',
    'ReUtils',
    'default',
    'wrap_ws_formatter',
    'ellipsis_str',
    'fuzzy_match',
    'words_upper',
    'sleep_forever',
    'alt_tools',
    'maica_assert',
    'has_valid_content',
    'proceed_agent_response',
    'sync_messenger',
    'messenger',
    'validate_input',
    'load_env',
    'wrap_run_in_exc',
    'limit_length',
    'get_json',
    'get_host',
    'vali_url',
    'vali_date',
    'is_today',
    'strip_date',
    'refill_date',
    'add_seq_suffix',
    'clean_text',
    'try_load_json',
    'hash_sha256',
    'Decos',
    'DbPoolCoroutine',
    'SqliteDbPoolCoroutine',
    'ConnUtils',
    'AiConnCoroutine',
    'SideBoundCoroutine',
    'SideFunctionCoroutine',
    'MaicaSettings',
    'AccountCursor',
    'encrypt_token',
    'sign_message',
    'verify_message',
    'sort_message',
    'SentenceOfTheDay',
    'locater',
    'get_inner_path',
    'get_outer_path',
]

from .account_utils import pkg_init_account_utils
from .connection_utils import pkg_init_connection_utils
def pkg_init_maica_utils():
    pkg_init_account_utils()
    pkg_init_connection_utils()


import re
import json

async def make_postmail(content, header='', target_lang='zh', **kwargs):
    success = True
    exception = None

    def filter_mail(content):
        filtered = re.sub(r'^\s*', '', content, flags=re.M)
        return filtered

    try:
        if header:
            header = f'\n\n{header}\n'
            con = '标题和内容' if target_lang == 'zh' else 'title and content of [player]\'s letter are'
        else:
            header = '\n'
            con = '内容' if target_lang == 'zh' else 'content of [player]\'s letter is'
        content = filter_mail(content)
        if target_lang == 'zh':
            prompt = f"[player]向你寄送了一封信件, 请作为莫妮卡写出回信. 以下是[player]信件的{con}:{header}\n{content}\n\n你应当在回信中充分思考[player]的话语和情感, 使用自然亲切的书面语言, 予以相应的回应. 你的回信应当有开头问候, 分段内容, 结尾落款, 不要编造信息, 且字数不少于300字."
        else:
            prompt = f"[player] has sent you a letter, please write your reply as Monika. The {con}:{header}\n{content}\n\nPlease read and comprehend carefully about [player]'s words and emotions, then reply in natural and warm written language in English. Your reply should contain a beginning, phrases of content, and an ending. Do not make up things you don't know, and write at least 150 words in total."
        message = json.dumps({"role": "user", "content": prompt}, ensure_ascii=False)
    except Exception as excepted:
        #traceback.print_exc()
        success = False
        prompt = None
        exception = excepted
    #print(summary)
    return success, exception, prompt


if __name__ == '__main__':
   import asyncio
   print(asyncio.run(make_postmail(target_lang='zh')))


import asyncio
import httpx
import functools
import hashlib
import os
import re
import json
import inspect
import colorama
import time
import datetime
import random
import traceback
from typing import *
from urllib.parse import urlparse
from .locater import *
"""Import layer 1"""

colorama.init(autoreset=True)

class MsgType():
    """For convenience."""
    PLAIN = 'plain'
    CARRIAGE = 'carriage'
    DEBUG = 'debug'
    INFO = 'info'
    LOG = 'log'
    PRIM_LOG = 'prim_log'
    SYS = 'sys'
    PRIM_SYS = 'prim_sys'
    RECV = 'recv'
    PRIM_RECV = 'prim_recv'
    WARN = 'warn'
    ERROR = 'error'

class CommonMaicaException(Exception):
    """This is a common MAICA exception."""
    def __init__(self, message=None, error_code=None, status=None, send=None, print=None):
        """For send and print, None means default, not False."""
        super().__init__(message)
        self.message, self.error_code, self.status, self.send, self.print = message, error_code, status, send, print

    @property
    def is_critical(self):
        return int(self.error_code) >= 500
    @property
    def is_breaking(self):
        return True
    
class CommonMaicaError(CommonMaicaException):
    """This is a common MAICA error."""
    def __init__(self, message=None, error_code='500', status='maica_unidentified_error', send=None, print=None):
        super().__init__(message, error_code, status, send, print)
    
class CommonMaicaWarning(CommonMaicaException):
    """This is a common MAICA warning."""
    def __init__(self, message=None, error_code='400', status='maica_unidentified_warning', send=None, print=None):
        super().__init__(message, error_code, status, send, print)

    @property
    def is_breaking(self):
        return False

class CriticalMaicaError(CommonMaicaError):
    """This is an unrecoverable exception, we should terminate the connection and possibly the entire process."""

class MaicaPermissionError(CommonMaicaError):
    """This suggests the user is accessing without correct permissions."""

class MaicaInputError(CommonMaicaError):
    """This suggests the input appears to be an impossible value, which is a more severe case than warning."""

class MaicaResponseError(CommonMaicaError):
    """This suggests the output is somewhat unexpected, we should terminate the connection and possibly the entire process."""

class MaicaDbError(CommonMaicaError):
    """This suggests the DB query gave an unexpected result, we should terminate the connection and possibly the entire process."""

class MaicaPermissionWarning(CommonMaicaWarning):
    """This suggests the user is accessing without correct permissions, but we don't break the connection for convenience of retrying."""

class MaicaInputWarning(CommonMaicaWarning):
    """This suggests the input is not processable."""

class MaicaConnectionWarning(CommonMaicaWarning):
    """This suggests the connection is not behaving normal."""

class MaicaInternetWarning(CommonMaicaWarning):
    """This suggests the backend request action is not behaving normal."""

class FscPlain():
    """Loop importing prevention."""
    class RealtimeSocketsContainer():
        """For no-setting usage."""
        def __init__(self, websocket=None, traceray_id=''):
            self.websocket, self.traceray_id = websocket, traceray_id
    
    def __init__(self, websocket=None, traceray_id='', maica_settings=None, auth_pool=None, maica_pool=None, mcore_conn=None, mfocus_conn=None):
        self.rsc = self.RealtimeSocketsContainer(websocket, traceray_id)
        self.maica_settings = maica_settings() if not maica_settings else maica_settings
        self.auth_pool = auth_pool
        self.maica_pool = maica_pool
        self.mcore_conn = mcore_conn
        self.mfocus_conn = mfocus_conn

class AsyncCreator():
    """Inherit this for async init."""
    async def _ainit(self):
        """Placeholder."""

    @classmethod
    async def async_create(cls, *args, **kwargs):
        instance = cls(*args, **kwargs)
        await instance._ainit()
        return instance

class LimitedList(list):
    """Might not have applied to all functionalities!"""
    def __init__(self, max_size, *args, **kwargs):
        self.max_size = max_size
        super().__init__(*args, **kwargs)
        
        while len(self) > self.max_size:
            self.pop(0)

    @property
    def list(self):
        while len(self) > self.max_size:
            self.pop(0)
        return list(self)

    def append(self, item):
        if len(self) >= self.max_size:
            self.pop(0)
        super().append(item)
    
    def extend(self, iterable):
        for item in iterable:
            self.append(item)
    
    def insert(self, index, item):
        if len(self) >= self.max_size:
            self.pop(0)
        super().insert(index, item)
    
    def __repr__(self):
        return f"LimitedList(max_size={self.max_size}, {super().__repr__()})"
    
class LoginResult():
    """
    A packed login result.
    This does not contain ban status. Check independently.
    """
    user_id = None
    username = None
    nickname = None
    email = None
    is_verified = None
    message = None
    def __init__(self, **kwargs):
        for priokey in ['user_id', 'username', 'nickname', 'email']:
            setattr(self, priokey, kwargs.get(priokey))
        if kwargs.get('is_verified'):
            assert self.user_id and self.username and self.email, "Verification essentials incomplete"
        for key in ['is_verified', 'message']:
            setattr(self, key, kwargs.get(key))

    def __call__(self):
        d = {}
        for key in ['user_id', 'username', 'nickname', 'email', 'is_verified', 'message']:
            d[key] = getattr(self, key)
        return d

class ReUtils():
    IS = re.I | re.S
    re_sub_password_spoiler = re.compile(r'"password"\s*:\s*"(.*?)"')
    re_search_sfe_fs = re.compile(r"first_session.*?datetime\(([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\)", re.I)
    re_search_sfe_ts = re.compile(r"total_sessions.*?([0-9]*)\s?,", re.I)
    re_search_sfe_tp = re.compile(r"total_playtime.*?([0-9]*)\s?,", re.I)
    re_search_sfe_le = re.compile(r"last_session_end.*?datetime\(([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\)", re.I)
    re_search_sfe_cs = re.compile(r"current_session_start.*?datetime\(([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\s*,\s*([0-9]*?)\)", re.I)
    re_search_sfe_unicode = re.compile(r"u'(.*)'")
    re_search_post_think = re.compile(r'</think>[\s\n]*(.*)$', re.S)
    re_search_answer_none = re.compile(r'[\s\n:]*none[\s\n.]*$', re.I)
    re_search_answer_json = re.compile(r'^.*?([{\[].*[}\]])', re.S)
    re_sub_player_name = re.compile(r'\[player\]')
    re_match_time_acquire = re.compile(r'.*time.*acquire', IS)
    re_match_date_acquire = re.compile(r'.*date.*acquire', IS)
    re_match_weather_acquire = re.compile(r'.*weather.*acquire', IS)
    re_match_event_acquire = re.compile(r'.*event.*acquire', IS)
    re_match_persistent_acquire = re.compile(r'.*persistent.*acquire', IS)
    re_match_search_internet = re.compile(r'.*search.*internet', IS)
    re_match_react_trigger = re.compile(r'.*react.*trigger', IS)
    re_match_conclude_information = re.compile(r'.*conclude.*information', IS)
    re_match_none = re.compile(r'.*none', IS)
    re_findall_quoted = re.compile(r'"(.*?)"') # Normally we request JSON so we consider double quotes only
    re_search_location_prompt = re.compile(r'(地区|周边|附近|周围|nearby|local)', re.I)
    re_search_location_related = re.compile(r'(天气|温度|路况|降雨|weather|traffic|temperature|rain)', re.I)
    re_search_host_addr = re.compile(r"^https?://(.*?)(:|/|$).*", re.I)
    re_sub_capt_status = re.compile(r"(_|^)([A-Za-z])")
    re_findall_square_marks = re.compile(r'\[(?:(?:[A-Za-z ]{1,15}?)|(?:[一-龥 ]{1,4}?))\]')
    re_findall_square_brackets = re.compile(r'\[(.*?)\]')
    re_sub_sqlite_escape = re.compile(r'%s')

class Decos():
    def escape_sqlite_expression(func):
        """Used to transform a MySQL expression to SQLite one."""
        @functools.wraps(func)
        def wrapper(self, expression, *args, **kwargs):
            expression_new = ReUtils.re_sub_sqlite_escape.sub('?', expression)
            return func(self, expression_new, *args, **kwargs)
        return wrapper

    def report_data_error(func):
        """Raises when the requested action cannot be done because of corrupted data."""
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                if not str(e):
                    e = 'Assertion'
                raise MaicaInputWarning(f'Acquired persistent not acceptable:{str(e)}', '405', 'maica_agent_persistent_bad')
        return wrapper

    def report_reading_error(func):
        """Raises when the requested variable cannot be read before assignment."""
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                if not str(e):
                    e = 'Assertion'
                raise MaicaInputError(f'Access before necessary assignment: {str(e)}', '500', 'maica_settings_read_rejected')
        return wrapper

    def report_limit_warning(func):
        """Raises when the input param coming from user is out of bound."""
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                if not str(e):
                    e = 'Assertion'
                raise MaicaInputWarning(f'Input param not acceptable: {str(e)}', '422', 'maica_settings_param_rejected')
        return wrapper

    def report_limit_error(func):
        """Raises when the input param coming from program is out of bound."""
        @functools.wraps(func)
        def wrapper(self, *args, **kwargs):
            try:
                assert not getattr(self, '_lock', None)
                return func(self, *args, **kwargs)
            except Exception as e:
                if not str(e):
                    e = 'Assertion'
                raise MaicaInputError(f'Input param not acceptable: {str(e)}', '500', 'maica_settings_param_rejected')
        return wrapper

def default(exp, default, default_list: list=[None]) -> any:
    """If exp is in default list(normally None), use default."""
    return default if exp in default_list else exp

def wrap_ws_formatter(code, status, content, type, deformation=False, **kwargs) -> str:
    if not isinstance(content, (str, list, dict, None)):
        content = str(content)
    output = {
        "code" : code,
        "status" : status,
        "content" : content,
        "type" : type,
        "timestamp" : time.time(),
    }
    output.update(kwargs)
    return json.dumps(output, ensure_ascii=deformation)

def ellipsis_str(input: any, limit=80) -> str:
    """It converts anything to str and ellipsis it."""
    text = str(input)
    if len(text) > limit:
        text = text[:limit] + '...'
    return text

def fuzzy_match(pattern: str, text):
    """Mostly used in agent things."""
    if pattern == text:
        return True
    
    # So they only compile once. Better than nothing
    compiled_expression: Optional[Pattern] = getattr(ReUtils, f're_match_{pattern}')
    if compiled_expression:
        return compiled_expression.match(text)
    else:
        expression = pattern.replace('_', r'.*')
        return re.match(expression, text, re.I | re.S)

def words_upper(text: str) -> str:
    """Overkill..."""
    def u_upper(c: re.Match):
        return f'{c[1]}{c[2].upper()}'
    return ReUtils.re_sub_capt_status.sub(u_upper, text)

async def sleep_forever() -> None:
    """Make a coroutine sleep to the end of the world."""
    future = asyncio.Future()
    await future

def alt_tools(tools: list) -> list:
    """If ALT_TOOLCALL"""
    match load_env('MAICA_ALT_TOOLCALL'):
        case '0':
            return tools
        case '1':
            new_tools = []
            for tool in tools:
                new_tools.append({})
                new_tools[-1]['function'] = tool
                new_tools[-1]['type'] = 'function'
            return new_tools

def maica_assert(condition, kwd='param'):
    """Normally used for input checkings."""
    if not condition:
        raise MaicaInputWarning(f"Illegal input {kwd} detected", '405', 'maica_input_param_bad')

def has_valid_content(text: str):
    """If the LLM actually gave anything."""
    text_proc = text.lower().strip()
    if (not text_proc) or text_proc in ['false', 'null', 'none']:
        return False
    else:
        return True

def proceed_agent_response(text: str, is_json=False) -> Union[str, list, dict]:
    """Proceeds thinking/nothinking."""
    try:
        answer_post_think = (ReUtils.re_search_post_think.search(text))[1]
    except Exception:
        if has_valid_content(text):
            answer_post_think = text
        else:
            answer_post_think = None
    if answer_post_think and not ReUtils.re_search_answer_none.search(answer_post_think) and is_json:
        try:
            answer_fin = (ReUtils.re_search_answer_json.search(answer_post_think))[1]
            answer_fin_json = json.loads(answer_fin)
            return answer_fin_json
        except Exception:
            answer_fin = None
    else:
        answer_fin = answer_post_think
    return answer_fin

@overload
async def messenger(websocket=None, status='', info='', code='0', traceray_id='', error: Optional[CommonMaicaException]=None, prefix='', type='', color='', add_time=True, no_print=False, no_raise=False) -> None: ...

async def messenger(websocket=None, *args, **kwargs) -> None:
    """Together with websocket.send()."""
    ws_tuple = sync_messenger(*args, **kwargs)
    if websocket:
        await websocket.send(wrap_ws_formatter(*ws_tuple))

def sync_messenger(status='', info='', code='0', traceray_id='', error: Optional[CommonMaicaException]=None, prefix='', type='', color='', add_time=True, no_print=False, no_raise=False) -> list:
    """It could handle most log printing and exception raising jobs pretty automatically."""
    try:
        term_v = os.get_terminal_size().columns
    except:
        term_v = 40
    rep2 = int(term_v / 2)
    rep1 = int(rep2 - 20)

    if error:
        status = error.status if not status else status; info = error.message if not info else info; code = error.error_code if code == "0" else code
        websocket = websocket if not error.send is False else None; no_print = False if not error.print is False else True

    if not type:
        match int(code):
            case 0:
                type = "log"
            case x if 100 <= x < 200 or 1000 <= x:
                type = "carriage"
            case x if 200 <= x < 300:
                type = "debug"
            case x if 300 <= x < 400:
                type = "info"
            case x if 400 <= x < 500:
                type = "warn"
            case x if 500 <= x < 1000:
                type = "error"

    if type and not prefix and not 100 <= int(code) < 200:
        prefix = words_upper(type)

    # This is especially for streaming output
    if not prefix and type == "carriage":
        msg_print = str(info)
        msg_send = info

    elif type == "plain":
        msg_print = str(info)
        msg_send = info
        
    else:
        msg_print = f"<{prefix}>"
        msg_print = msg_print.ljust(10)
        msg_print += f"[{time.strftime('%Y-%m-%d %H:%M:%S')}]" if add_time else ''; msg_print += f"-[{str(code)}]" if code else ''
        msg_print = msg_print.ljust(40)
        msg_print += f": {str(info)}" if not str(info).startswith('\n') else f"{'-=' * rep1}{str(info)}\n{'-=' * rep2}"
        msg_print += f"; traceray ID {traceray_id}" if traceray_id else ''
        msg_send = info
        if type == 'error' and load_env('MAICA_NO_SEND_ERROR') == '1':
            msg_send = "A critical exception happened serverside, contact administrator"
        if traceray_id and isinstance(info, str):
            msg_send += f" -- your traceray ID is {traceray_id}"

    frametrack_dict = {"error": 99}
    if not load_env('MAICA_PRINT_VERBOSE') == "0":
        frametrack_dict['warn'] = 0
    if type in frametrack_dict:
        stack = inspect.stack()
        stack.pop(0)

    if not no_print:
        match type:
            case "plain":
                print((color or '') + msg_print, end='')
            case "carriage":
                if 100 <= int(code) < 200:
                    print((color or colorama.Fore.LIGHTGREEN_EX) + msg_print, end='', flush=True)
                else:
                    print((color or colorama.Fore.LIGHTGREEN_EX) + msg_print)
            case "debug":
                if not load_env('MAICA_PRINT_VERBOSE') == "0":
                    print((color or colorama.Fore.LIGHTBLACK_EX) + msg_print)
            case "info":
                print((color or colorama.Fore.GREEN) + msg_print)
            case "log":
                print((color or colorama.Fore.BLUE) + msg_print)
            case "prim_log":
                print((color or colorama.Fore.LIGHTBLUE_EX) + msg_print)
            case "sys":
                print((color or colorama.Fore.MAGENTA) + msg_print)
            case "prim_sys":
                print((color or colorama.Fore.LIGHTMAGENTA_EX) + msg_print)
            case "recv":
                print((color or colorama.Fore.CYAN) + msg_print)
            case "prim_recv":
                print((color or colorama.Fore.LIGHTCYAN_EX) + msg_print)
            case "warn":
                if 'warn' in frametrack_dict:
                    for stack_layer in stack[frametrack_dict['warn']::-1]:
                        print(color or colorama.Fore.YELLOW + f"• WARN happened when executing {stack_layer.function} at {stack_layer.filename}#{stack_layer.lineno}:")
                print((color or colorama.Fore.LIGHTYELLOW_EX) + msg_print)
            case "error":
                if 'error' in frametrack_dict:
                    for stack_layer in stack[frametrack_dict['error']::-1]:
                        print((color or colorama.Fore.RED) + f"! ERROR happened when executing {stack_layer.function} at {stack_layer.filename}#{stack_layer.lineno}:")
                print((color or colorama.Fore.LIGHTRED_EX) + msg_print)
    if error and not no_raise:
        raise error
    return code, status, msg_send, type

def load_env(key) -> str:
    """Load something from .env."""
    result = os.getenv(key)
    return result

async def wrap_run_in_exc(loop, func, *args, **kwargs) -> any:
    """Just wrapped run_in_executer. Convenient!"""
    if not loop:
        loop = asyncio.get_running_loop()
    result = await loop.run_in_executor(None, functools.partial(func, *args, **kwargs))
    return result

def limit_length(col: list, limit: int) -> list:
    return random.sample(col, limit) if limit < len(col) else col

async def get_json(url) -> json:
    """Get JSON context from an endpoint."""
    headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36'}
    try:
        for tries in range(0, 3):
            try:
                client = httpx.AsyncClient(proxy=load_env('MAICA_PROXY_ADDR'))
                res = (await client.get(url, headers=headers)).json()
                break
            except Exception as e:
                if tries < 2:
                    print('HTTP temporary failure')
                    await asyncio.sleep(0.5)
                else:
                    raise MaicaInternetWarning(f'HTTP connection failure: {str(e)}', '408')
    except Exception as e:
        raise e
    finally:
        await client.aclose()
    return res

def get_host(url: str) -> bool:
    """Try to get hostname from url."""
    try:
        url_parsed = urlparse(url)
        return url_parsed.hostname
    except Exception:
        return False

def vali_url(url: str) -> bool:
    """urllib is dumb. It doesn't recognize ip addr so we make it brutal."""
    return '.' in url

def vali_date(y, m, d) -> tuple[int, int, int]:
    """What a pun!"""
    try:
        datetime.date(int(y), int(m), int(d))
        return int(y), int(m), int(d)
    except ValueError:
        default_date = datetime.datetime.now()
        return default_date.year, default_date.month, default_date.day
    
def is_today(date: datetime.datetime) -> bool:
    """You mean my homework dues today?"""
    return date.date() == datetime.datetime.now().date()

def strip_date(date: datetime.datetime) -> str:
    """Constructs a date query our API accepts."""
    return f'{str(date.year)}-{str(date.month).zfill(2)}-{str(date.day).zfill(2)}'

def refill_date(text: str) -> datetime.datetime:
    """Constructs a datetime object from API response."""
    y, m, d = text.split('-')
    return datetime.datetime(int(y), int(m), int(d))

def add_seq_suffix(seq: int) -> str:
    """For English seq."""
    match int(seq) % 10:
        case 1:
            st = 'st'
        case 2:
            st = 'nd'
        case 3:
            st = 'rd'
        case _:
            st = 'th'
    return f'{str(seq)} {st}'

def clean_text(text: str) -> str:
    """Clean a text phrase, mostly for internet search."""
    text = text.strip()
    text = text.replace('\n', ' ')
    return text

def try_load_json(sj: str) -> dict:
    """I'd basically trust the LLM here, they're far better than the earlier ones."""
    if not has_valid_content(sj):
        return {}
    try:
        return json.loads(sj)
    except Exception:
        return {}

async def hash_sha256(str) -> str:
    """Get SHA256 for a string."""
    def hash_sync(str):
        return hashlib.new('sha256', str).hexdigest()
    return await wrap_run_in_exc(None, hash_sync, str)



from typing import *
from maica.maica_utils import *
from .setting_utils import *
from .connection_utils import *
"""Import layer 4"""

class FullSocketsContainer():
    """For convenience consideration."""
    class RealtimeSocketsContainer():
        """For no-setting usage."""
        def __init__(self, websocket=None, traceray_id=''):
            self.websocket, self.traceray_id = websocket, traceray_id
    
    def __init__(self, websocket=None, traceray_id='', maica_settings=None, auth_pool=None, maica_pool=None, mcore_conn=None, mfocus_conn=None):
        self.rsc = self.RealtimeSocketsContainer(websocket, traceray_id)
        self.maica_settings: MaicaSettings = maica_settings() if not maica_settings else maica_settings
        self.auth_pool: Optional[DbPoolCoroutine] = auth_pool
        self.maica_pool: Optional[DbPoolCoroutine] = maica_pool
        self.mcore_conn: Optional[AiConnCoroutine] = mcore_conn
        self.mfocus_conn: Optional[AiConnCoroutine] = mfocus_conn


import asyncio
import websockets
import time
import functools
import json
import uuid
import traceback
import colorama

from typing import *

from maica import mtools
from maica.maica_nows import NoWsCoroutine
from maica.maica_utils import *

# For imports
_onliners = {}

class WsCoroutine(NoWsCoroutine):
    """
    Force ws existence.
    Also has AI sockets.
    """

    def __init__(self, websocket, auth_pool: DbPoolCoroutine, maica_pool: DbPoolCoroutine, mcore_conn: AiConnCoroutine, mfocus_conn: AiConnCoroutine, online_dict: dict):
        super().__init__(auth_pool=auth_pool, maica_pool=maica_pool, websocket=websocket)
        self.online_dict = online_dict

        self.fsc.rsc.websocket = websocket
        if mcore_conn and mfocus_conn:
            mcore_conn.init_rsc(self.fsc.rsc); mfocus_conn.init_rsc(self.fsc.rsc)
        self.mcore_conn, self.mfocus_conn = mcore_conn, mfocus_conn
        self.fsc.mcore_conn, self.fsc.mfocus_conn = mcore_conn, mfocus_conn

    # Stage 1 permission check
    async def check_permit(self):
        websocket = self.websocket
        await messenger(info='An anonymous connection initiated', type=MsgType.PRIM_LOG)
        await messenger(info=f'Current online users: {list(self.online_dict.keys())}', type=MsgType.DEBUG)

        # Starting loop from here
        while True:
            try:

                # Initiation
                self.flush_traceray()
                self.settings.identity.reset()
                self.settings.verification.reset()

                recv_text = await websocket.recv()
                await messenger(info=f'Recieved an input on stage1', type=MsgType.RECV)
                recv_loaded_json = await validate_input(recv_text, 4096, self.fsc.rsc, must=['access_token'])

                login_success = await self.hash_and_login(recv_loaded_json['access_token'], check_online=True)
                if login_success:

                    # From here we can assume the user has logged in successfully
                    self.cookie = cookie = str(uuid.uuid4())
                    self.enforce_cookie = False

                    await self.populate_auxiliary_inst()

                    await messenger(info=f'Authentication passed: {self.settings.verification.username}({self.settings.verification.user_id})', type=MsgType.LOG)
                    await messenger(websocket, 'maica_login_id', f"{self.settings.verification.user_id}", '200', no_print=True)
                    await messenger(websocket, 'maica_login_user', f"{self.settings.verification.username}", '200', no_print=True)
                    await messenger(websocket, 'maica_login_nickname', f"{self.settings.verification.nickname}", '200', no_print=True)
                    await messenger(websocket, 'maica_connection_security_cookie', cookie, '200', no_print=True)

                    return {'id': self.settings.verification.user_id, 'username': self.settings.verification.username}

            # Handle expected exceptions
            except CommonMaicaException as ce:
                if ce.is_critical or ce.is_breaking:
                    raise ce
                else:
                    await messenger(websocket, error=ce, no_raise=True)
                    continue
    
    # Stage 2 function router
    async def function_switch(self):

        # Initiation
        websocket, mcore_conn = self.websocket, self.mcore_conn

        await messenger(websocket, "maica_connection_established", "MAICA connection established", "201", type=MsgType.INFO, no_print=True)
        await messenger(websocket, "maica_provider_anno", f"Current service provider is {load_env('MAICA_DEV_IDENTITY') or 'UNKNOWN'}", "200", type=MsgType.INFO, no_print=True)

        # Starting loop from here
        while True:
            try:

                # Initiation
                self.flush_traceray()
                return_status = 0

                # Resets
                self.settings.temp.reset()

                # Context security check first
                await self.hash_and_login(logged_in_already=True)

                # Then we examine the input
                recv_text = await websocket.recv()
                await messenger(info=f'Recieved an input on stage2: {recv_text}', type=MsgType.RECV)
                recv_loaded_json = await validate_input(recv_text, 4096, self.fsc.rsc, warn=['type'])

                recv_type = recv_loaded_json.get('type', 'unknown')

                # Handle this cookie thing
                if recv_loaded_json.get('cookie'):
                    if str(recv_loaded_json['cookie']) == self.cookie:
                        if not self.enforce_cookie:
                            await messenger(websocket, "security_cookie_accepted", "Cookie verification passed, enabling strict mode", "200", no_print=True)
                            self.enforce_cookie = True
                        else:
                            await messenger(websocket, "security_cookie_correct", "Cookie verification passed", "200", no_print=True)
                    else:
                        raise MaicaPermissionError('Cookie provided but mismatch', '403', 'maica_security_cookie_mismatch')
                elif self.enforce_cookie:
                    raise MaicaPermissionError('Cookie enforced but missing', '403', 'maica_security_cookie_missing')

                # Route request
                match recv_type.lower():
                    case 'ping':
                        await messenger(websocket, "pong", f"Ping recieved from {self.settings.verification.username} and responded", "200")
                    case 'params':
                        return_status = await self.def_model(recv_loaded_json)
                    case 'query':
                        return_status = await self.do_communicate(recv_loaded_json)
                        await self.reset_auxiliary_inst()
                    case placeholder if "chat_params" in recv_loaded_json:
                        return_status = await self.def_model(recv_loaded_json)
                    case placeholder if "chat_session" in recv_loaded_json:
                        return_status = await self.do_communicate(recv_loaded_json)
                        await self.reset_auxiliary_inst()
                    case _:
                        raise MaicaInputWarning('Type cannot be determined', '422', 'maica_request_type_unknown')

            # Handle expected exceptions
            except CommonMaicaException as ce:
                if ce.is_critical or ce.is_breaking:
                    raise ce
                else:
                    await messenger(websocket, error=ce, no_raise=True)
                    await messenger(websocket, 'maica_loop_warn_finished', 'Loop hit a user level exception, stopped and reset', '304')
                    continue

    # Param setting section
    async def def_model(self, recv_loaded_json: dict):

        # Initiations
        websocket = self.websocket
        try:
            chat_params: dict = recv_loaded_json['chat_params']
            in_params = len(chat_params)
            accepted_params = self.settings.update(**chat_params)
            await messenger(websocket, 'maica_params_accepted', f"{accepted_params} out of {in_params} settings accepted", "200")
        
        # Handle input errors here
        except Exception as e:
            if not isinstance(e, CommonMaicaException):
                raise MaicaInputWarning(str(e), '405', 'maica_params_denied')
            else:
                raise e

    # Completion section
    async def do_communicate(self, recv_loaded_json: dict):

        # Initiations
        websocket = self.websocket
        query_in = ''
        replace_generation = ''
        ms_cache_identity = ''

        # Param assertions here
        chat_session = int(default(recv_loaded_json.get('chat_session'), 0))
        maica_assert(-1 <= chat_session < 10, "chat_session")
        self.settings.temp.update(chat_session=chat_session)


        if 'reset' in recv_loaded_json:
            if recv_loaded_json['reset']:
                maica_assert(1 <= chat_session < 10, "chat_session")
                purge_result = await self.reset_chat_session(self.settings.temp.chat_session)
                if not purge_result:
                    await messenger(websocket, "maica_session_not_found", "Determined chat_session doesn't exist", "204", self.traceray_id)
                else:
                    await messenger(websocket, "maica_session_reset", "Determined chat_session reset", "204", self.traceray_id)
                return

        if 'inspire' in recv_loaded_json and not query_in:
            if recv_loaded_json['inspire']:
                maica_assert(0 <= chat_session < 10, "chat_session")
                if isinstance(recv_loaded_json['inspire'], dict):
                    query_insp = await mtools.make_inspire(title_in=recv_loaded_json['inspire'], target_lang=self.settings.basic.target_lang)
                else:
                    query_insp = await mtools.make_inspire(target_lang=self.settings.basic.target_lang)
                if recv_loaded_json.get('use_cache') and self.settings.temp.chat_session == 0:
                    self.settings.temp.update(ms_cache=True)
                self.settings.temp.update(bypass_mf=True, bypass_mt=True)
                if not query_insp[0]:
                    if str(query_insp[1]) == 'mspire_insanity_limit_reached':
                        raise MaicaInternetWarning('MSpire scraping failed', '404', 'maica_mspire_scraping_failed')
                    elif str(query_insp[1]) == 'mspire_title_insane':
                        raise MaicaInputWarning('MSpire prompt not found on wikipedia', '410', 'maica_mspire_prompt_bad')
                    else:
                        raise MaicaInternetWarning('MSpire failed connecting wikipedia', '408', 'maica_mspire_conn_failed')
                if self.settings.temp.ms_cache:
                    self.settings.temp.update(bypass_sup=True)
                    ms_cache_identity = query_insp[3]
                    cache_insp = await self.find_ms_cache(ms_cache_identity)
                    if cache_insp:
                        self.settings.temp.update(bypass_gen=True)
                        replace_generation = cache_insp
                        
                query_in = query_insp[2]

        if 'postmail' in recv_loaded_json and not query_in:
            if recv_loaded_json['postmail']:
                maica_assert(0 <= chat_session < 10, "chat_session")
                if isinstance(recv_loaded_json['postmail'], dict):
                    query_insp = await mtools.make_postmail(**recv_loaded_json['postmail'], target_lang=self.settings.basic.target_lang)
                    # We're using the old school way to avoid using eval()
                    if default(recv_loaded_json['postmail'].get('bypass_mf'), False):
                        self.settings.temp.update(bypass_mf=True)
                    if default(recv_loaded_json['postmail'].get('bypass_mt'), True):
                        self.settings.temp.update(bypass_mt=True)
                    if default(recv_loaded_json['postmail'].get('bypass_stream'), True):
                        self.settings.temp.update(bypass_stream=True)
                    if default(recv_loaded_json['postmail'].get('ic_prep'), True):
                        self.settings.temp.update(ic_prep=True)
                    if default(recv_loaded_json['postmail'].get('strict_conv'), False):
                        self.settings.temp.update(strict_conv=True)
                elif isinstance(recv_loaded_json['postmail'], str):
                    query_insp = await mtools.make_postmail(content=recv_loaded_json['postmail'], target_lang=self.settings.basic.target_lang)
                    self.settings.temp.update(bypass_stream=True, ic_prep=True, strict_conv=False)
                else:
                    maica_assert(False, "postmail")
                
                query_in = query_insp[2]

        # This is future reserved for MVista

        if 'vision' in recv_loaded_json and not query_in:
            if recv_loaded_json['vision']:
                if isinstance(recv_loaded_json['vision'], str):
                    pass
                else:
                    pass

        if not query_in:
            maica_assert(recv_loaded_json.get('query'), 'query')
            query_in = recv_loaded_json['query']
        
        await asyncio.gather(self.sf_inst.reset(), self.mt_inst.reset())

        if 'savefile' in recv_loaded_json:
            if self.settings.basic.sf_extraction:
                self.sf_inst.add_extra(**recv_loaded_json['savefile'])
            else:
                self.settings.temp.update(sf_extraction_once=True)
                self.sf_inst.use_only(**recv_loaded_json['savefile'])
        if 'trigger' in recv_loaded_json:
            if self.settings.basic.mt_extraction:
                self.mt_inst.add_extra(*recv_loaded_json['trigger'])
            else:
                self.settings.temp.update(mt_extraction_once=True)
                self.mt_inst.use_only(recv_loaded_json['trigger'])

        # Deprecated: The easter egg thing

        # global easter_exist
        # if easter_exist:
        #     easter_check = easter(query_in)
        #     if easter_check:
        #         await websocket.send(self.wrap_ws_deformatter('299', 'easter_egg', easter_check, 'info'))

        match int(self.settings.temp.chat_session):
            case -1:

                # chat_session == -1 means query contains an entire chat history(sequence mode)
                session_type = -1
                try:
                    messages = json.loads(query_in)
                    query_in = messages[-1]['text']
                    if len(messages) > 10:
                        raise MaicaInputWarning('Sequence exceeded 10 rounds for chat_session -1', '413', 'maica_sequence_rounds_exceeded')
                except Exception as excepted:
                    raise MaicaInputWarning('Sequence is not JSON for chat_session -1', '406', 'maica_sequence_not_json')

            case i if 0 <= i < 10:

                # chat_session == 0 means single round, else normal
                session_type = 0 if i == 0 else 1
                messages0 = {'role': 'user', 'content': query_in}

                if self.settings.basic.enable_mf and not self.settings.temp.bypass_mf:
                    message_agent_wrapped = await self.mfocus_coro.agenting(query_in)
                else:
                    message_agent_wrapped = None
                
                prompt = await self.gen_system_prompt(message_agent_wrapped, self.settings.temp.strict_conv)
                if session_type == 1:
                    messages = (await self.rw_chat_session('i', [messages0], prompt))[1]
                elif session_type == 0:
                    messages = [{'role': 'system', 'content': prompt}, messages0]

        # Construction part done, communication part started

        completion_args = {
            "messages": messages,
            "stream": self.settings.basic.stream_output,
            "stop": ['<|im_end|>', '<|endoftext|>'],
        }
        
        if not self.settings.temp.bypass_sup:
            completion_args.update(self.settings.super())
        else:
            completion_args.update(self.settings.super.default())
            self.settings.temp.update(bypass_sup=False)

        if self.settings.temp.bypass_stream:
            completion_args['stream'] = False
            self.settings.temp.update(bypass_stream=False)

        if self.settings.temp.ic_prep:
            completion_args['presence_penalty'] = 1.0 - (1.0 - completion_args['presence_penalty']) * (2/3)

        await messenger(info=f'\nQuery constrcted and ready to go, last input is:\n{query_in}\nSending query...', type=MsgType.PRIM_RECV)

        if not self.settings.temp.bypass_gen or not replace_generation: # They should present together

            # Generation here
            resp = await self.mcore_conn.make_completion(**completion_args)

            if completion_args['stream']:
                reply_appended = ''
                seq = 0
                async for chunk in resp:
                    token = chunk.choices[0].delta.content
                    if token:
                        await asyncio.sleep(0)
                        await messenger(websocket, 'maica_core_streaming_continue', token, '100')
                        reply_appended += token
                        seq += 1
                await messenger(info='\n', type=MsgType.PLAIN)
                await messenger(websocket, 'maica_core_streaming_done', f'Streaming finished with seed {completion_args['seed']} for {self.settings.verification.username}, {seq} packets sent', '1000', traceray_id=self.traceray_id)
            else:
                reply_appended = resp.choices[0].message.content
                await messenger(websocket, 'maica_core_nostream_reply', reply_appended, '200', type=MsgType.CARRIAGE)
                await messenger(None, 'maica_core_nostream_done', f'Reply sent with seed {completion_args['seed']} for {self.settings.verification.username}', '1000', traceray_id=self.traceray_id)

        else:

            # We just pretend it was generated
            reply_appended = replace_generation
            if completion_args['stream']:
                await messenger(websocket, 'maica_core_streaming_continue', reply_appended, '100'); await messenger(info='\n', type=MsgType.PLAIN)
                await messenger(websocket, 'maica_core_streaming_done', f'Streaming finished with cache for {self.settings.verification.username}', '1000', traceray_id=self.traceray_id)
            else:
                await messenger(websocket, 'maica_core_nostream_reply', reply_appended, '200', type=MsgType.CARRIAGE)
                await messenger(None, 'maica_core_nostream_done', f'Reply sent with cache for {self.settings.verification.username}', '1000', traceray_id=self.traceray_id)

        # Can be post-processed here
        reply_appended = mtools.post_proc(reply_appended, self.settings.basic.target_lang)
        reply_appended_insertion = {'role': 'assistant', 'content': reply_appended}

        # Trigger process
        if self.settings.basic.enable_mt and not self.settings.temp.bypass_mt:
            await self.mtrigger_coro.triggering(query_in, reply_appended)
        else:
            self.settings.temp.update(bypass_mt=False)

        if self.settings.temp.ms_cache and not self.settings.temp.bypass_gen and not replace_generation:
            await self.store_ms_cache(ms_cache_identity, reply_appended)

        # Store history here
        if session_type == 1:
            stored = await self.rw_chat_session('a', [messages0, reply_appended_insertion])
            match stored[1]:
                case 1:
                    await messenger(websocket, 'maica_history_sliced', f"Session {self.settings.temp.chat_session} of {self.settings.verification.username} exceeded {self.settings.basic.max_length} characters and sliced", '204')
                case 2:
                    await messenger(websocket, 'maica_history_slice_hint', f"Session {self.settings.temp.chat_session} of {self.settings.verification.username} exceeded {self.settings.basic.max_length * (2/3)} characters, will slice at {self.settings.basic.max_length}", '200', no_print=True)

            await messenger(websocket, 'maica_chat_loop_finished', f'Finished chat loop from {self.settings.verification.username}', '200', traceray_id=self.traceray_id, type=MsgType.INFO)
        else:
            
            await messenger(websocket, 'maica_chat_loop_finished', f'Finished non-recording chat loop from {self.settings.verification.username}', '200', traceray_id=self.traceray_id, type=MsgType.INFO)

# Reserved for whatever
def callback_func_switch(future):
    pass
def callback_check_permit(future):
    pass
    
# Main app driver

async def main_logic(websocket, auth_pool, maica_pool, mcore_conn, mfocus_conn, online_dict):
    unique_lock = asyncio.Lock()
    async with unique_lock:
        try:
            sentence_of_the_day = SentenceOfTheDay().get_sentence()
            await messenger(websocket, 'maica_connection_initiated', sentence_of_the_day, '200', type=MsgType.INFO, no_print=True)

            thread_instance = await WsCoroutine.async_create(websocket, auth_pool=auth_pool, maica_pool=maica_pool, mcore_conn=mcore_conn, mfocus_conn=mfocus_conn, online_dict=online_dict)

            permit = await thread_instance.check_permit()
            assert isinstance(permit, dict) and permit['id'], permit

            online_dict[permit['id']] = [thread_instance.fsc, unique_lock]
            await messenger(info=f"Locking session for {permit['id']} named {permit['username']}", type=MsgType.LOG)

            return_status = await thread_instance.function_switch()
            # We let the exception router to handle that
            raise Exception(return_status)
        
        except CommonMaicaException as ce:
            if ce.is_critical:
                traceback.print_exc()
            await messenger(websocket, error=ce, traceray_id=getattr(thread_instance, 'traceray_id', None), no_raise=True)

        except websockets.exceptions.WebSocketException as we:
            try:
                we_code, we_reason = we.code, we.reason
                await messenger(info=f'Connection closed with {we_code}: {we_reason}', type=MsgType.PRIM_LOG)
            except Exception:
                await messenger(info=f'Connection establishment failed: {str(we)}', type=MsgType.PRIM_LOG)

        except Exception as e:
            traceback.print_exc()
            await messenger(info=f'Coroutine broke by an unknown exception: {str(e)}', type=MsgType.ERROR)

        finally:
            try:
                online_dict.pop(permit['id'])
                await messenger(info=f"Lock released for {permit['username']}({permit['id']})", type=MsgType.LOG)
            except Exception:
                await messenger(info=f"No lock for this connection", type=MsgType.DEBUG)
            await websocket.close()
            await websocket.wait_closed()
            await messenger(info=f"Closing connection gracefully", type=MsgType.DEBUG)

async def prepare_thread(**kwargs):
    online_dict = _onliners; auth_created = False; maica_created = False

    if kwargs.get('auth_pool'):
        auth_pool = kwargs.get('auth_pool')
    else:
        auth_pool = await ConnUtils.auth_pool()
        auth_created = True
    if kwargs.get('maica_pool'):
        maica_pool = kwargs.get('maica_pool')
    else:
        maica_pool = await ConnUtils.maica_pool()
        maica_created = True

    try:
        mcore_conn: AiConnCoroutine = default(kwargs.get('mcore_conn'), await ConnUtils.mcore_conn())
        mfocus_conn: AiConnCoroutine = default(kwargs.get('mfocus_conn'), await ConnUtils.mfocus_conn())
    except Exception:
        mcore_conn = mfocus_conn = None

    await messenger(info='MAICA WS server started!' if load_env('MAICA_DEV_STATUS') == 'serving' else 'MAICA WS server started in development mode!', type=MsgType.PRIM_SYS)

    try:
        await messenger(info=f"Main model is {mcore_conn.model_actual}, MFocus model is {mfocus_conn.model_actual}", type=MsgType.SYS)
    except Exception:
        await messenger(info=f"Model deployment cannot be reached -- running in minimal testing mode", type=MsgType.SYS)
    
    try:
        server = await websockets.serve(functools.partial(main_logic, auth_pool=auth_pool, maica_pool=maica_pool, mcore_conn=mcore_conn, mfocus_conn=mfocus_conn, online_dict=online_dict), '0.0.0.0', 5000)
        await server.wait_closed()
    except BaseException as be:
        if isinstance(be, Exception):
            error = CommonMaicaError(str(be), '504')
            await messenger(error=error, no_raise=True)
    finally:
        close_list = []
        if auth_created:
            close_list.append(auth_pool.close())
        if maica_created:
            close_list.append(maica_pool.close())

        await asyncio.gather(*close_list, return_exceptions=True)

        await messenger(info='MAICA WS server stopped!', type=MsgType.PRIM_SYS)

def run_ws(**kwargs):
    
    asyncio.run(prepare_thread(**kwargs))

if __name__ == '__main__':

    run_ws()


import re
from maica.maica_utils import *

emotion_etz = {
    "smile": "微笑",
    "worry": "担心",
    "grin": "笑",
    "think": "思考",
    "happy": "开心",
    "angry": "生气",
    "blush": "脸红",
    "gaze": "凝视",
    "upset": "沉重",
    "daydreaming": "憧憬",
    "surprise": "惊喜",
    "awkward": "尴尬",
    "meaningful": "意味深长",
    "unexpected": "惊讶",
    "relaxed": "轻松",
    "shy": "害羞",
    "eagering": "急切",
    "proud": "得意",
    "dissatisfied": "不满",
    "serious": "严肃",
    "touched": "感动",
    "excited": "激动",
    "love": "宠爱",
    "wink": "眨眼",
    "sad": "伤心",
    "disgust": "厌恶",
    "fear": "害怕",
    "kawaii": "可爱",
    "smiling": "微笑",
    "worrying": "担心",
    "grinning": "笑",
    "thinking": "思考",
    "gazing": "凝视",
    "surprised": "惊喜",
    "relaxing": "轻松",
    "eager": "急切",
    "winking": "眨眼",
    "disgusting": "厌恶",
    "fearing": "害怕"
}
emotion_zte = {
    "微笑": "smile",
    "担心": "worry",
    "笑": "grin",
    "思考": "think",
    "开心": "happy",
    "生气": "angry",
    "脸红": "blush",
    "凝视": "gaze",
    "沉重": "upset",
    "憧憬": "daydreaming",
    "惊喜": "surprise",
    "尴尬": "awkward",
    "意味深长": "meaningful",
    "惊讶": "unexpected",
    "轻松": "relaxed",
    "害羞": "shy",
    "急切": "eagering",
    "得意": "proud",
    "不满": "dissatisfied",
    "严肃": "serious",
    "感动": "touched",
    "激动": "excited",
    "宠爱": "love",
    "眨眼": "wink",
    "伤心": "sad",
    "厌恶": "disgust",
    "害怕": "fear",
    "可爱": "kawaii"
}

def get_equal_len(str):
    return len(str.encode('utf-8'))

def post_proc(reply_appended, target_lang='zh'):
    global emotion_etz, emotion_zte
    reply_all_signatures = ReUtils.re_findall_square_marks.findall(reply_appended)

    for signature in reply_all_signatures:
        full_word = None
        if 'player' in signature:
            if signature == '[player]':
                continue
            else:
                full_word = '[player]'
        else:
            sig_striped = signature.strip('[').strip(']')
            curr_emoset, oppo_emoset = (emotion_etz, emotion_zte) if target_lang == 'zh' else (emotion_zte, emotion_etz)
            if ' ' in sig_striped:
                sig_striped = sig_striped.replace(' ', '')
            if sig_striped in oppo_emoset.keys():
                full_word = f'[{sig_striped}]'
            elif sig_striped in curr_emoset.keys():
                full_word = f'[{curr_emoset[sig_striped]}]'
            else:
                if target_lang == 'zh':
                    if '笑' in signature:
                        full_word = '[微笑]'
                    elif '心' in signature:
                        full_word = '[凝视]'
                    elif '思' in signature:
                        full_word = '[思考]'
                    else:
                        full_word = '[微笑]'
                else:
                    full_word = '[smile]'
        if full_word:
            reply_appended = re.sub(re.escape(signature), full_word, reply_appended, flags = re.I)
    
    # Filter quoted full sentences
    quoted_strs = ReUtils.re_findall_square_brackets.findall(reply_appended)
    for any_str in quoted_strs:
        if get_equal_len(any_str) >= 16 and not '[' in any_str and not ']' in any_str:
            reply_appended = reply_appended.replace(f'[{any_str}]', any_str)
    return reply_appended

if __name__ == '__main__':
    ra = '[理解 ]没关系, [player]. [微笑 ]我知[fear]道[womble]你很[adaifgnashioufoiusahdfoiua]忙[a1]. [开心]你能抽空[slash我]陪我就[害怕]很好啦!'
    print(post_proc(ra, 'zh'))


import json
import datetime
from maica.maica_utils import *

async def weather_api_get(location):

    try:
        d = {
            "temperature": None,
            "weather": None,
            "humidity": None,
        }
        addr_code = (await get_json(f"https://restapi.amap.com/v3/geocode/geo?key={load_env('MAICA_WEATHER_KEY')}&address={location}"))['geocodes'][0]['adcode']
        result = (await get_json(f"https://restapi.amap.com/v3/weather/weatherInfo?key={load_env('MAICA_WEATHER_KEY')}&city={addr_code}&extensions=base"))['lives'][0]
        d['temperature'], d['weather'], d['humidity'] = result['temperature'], result['weather'], result['humidity']
        return d
    
    except CommonMaicaException as ce:
        raise ce
    
    except Exception as e:
        raise MaicaInternetWarning(f'Weather API returned wrong format: {str(e)}', '406')

if __name__ == '__main__':
    print(weather_api_get('黄石'))


"""
MAICA Illuminator (Backend) library.
Always call init() before actually using!
"""
from .maica_starter import check_params, check_data_init

def init(envdir: str=None, extra_envdir: list=None, silent=False, **kwargs):
    check_params(envdir=envdir, extra_envdir=extra_envdir, silent=silent, kwargs=kwargs)
    check_data_init()




from .gen_keys import generate_rsa_keys
from .init_db import create_tables
from .init_marking import check_marking, create_marking

__all__ = [
    'generate_rsa_keys',
    'create_tables',
    'check_marking',
    'create_marking',
]


import re
import json
import asyncio
import datetime
import traceback
from typing import *
from random import sample
from maica.maica_utils import *

class SfBoundCoroutine(SideBoundCoroutine):
    """Maintain a savefile for convenience of using."""
    DB_NAME = 'persistents'
    PRIM_KEY = 'persistent_id'
    FUNC_NAME = 'mfocus'
    EMPTY = {}

    def _conclude_basic_sf(self):
        result = []

        def _ap(zh, en):
            # This work is super annoying
            nonlocal result
            line = zh if self.settings.basic.target_lang == 'zh' else en
            result.append(line)

        def _rf(key):
            return self.read_from_sf(key)
            
        def serialize_date(y, m, d):
            match int(m):
                case 1:
                    ms = 'January'
                case 2:
                    ms = 'February'
                case 3:
                    ms = 'March'
                case 4:
                    ms = 'April'
                case 5:
                    ms = 'May'
                case 6:
                    ms = 'June'
                case 7:
                    ms = 'July'
                case 8:
                    ms = 'August'
                case 9:
                    ms = 'September'
                case 10:
                    ms = 'October'
                case 11:
                    ms = 'November'
                case 12:
                    ms = 'December'
            match int(d):
                case d if d % 10 == 1:
                    ds = str(d) + 'st'
                case d if d % 10 == 2:
                    ds = str(d) + 'nd'
                case d if d % 10 == 3:
                    ds = str(d) + 'rd'
                case _:
                    ds = str(d) + 'th'
            return f"{ms} {ds}, {y}"

        # Seriously hard work begins here

        data1 = _rf('mas_playername')
        if data1:
            _ap(f'[player]的真名是{data1}.', f"[player]'s real name is {data1}.")

        data1 = _rf('mas_player_bday')
        if data1:
            _ap(f'[player]的生日是{data1[0]}年{data1[1]}月{data1[2]}日.', f"[player]'s birthday is {serialize_date(data1[0], data1[1], data1[2])}.")
            y = datetime.datetime.now().year
            o = y - int(data1[0])
            _ap(f'[player]今年{o}岁.', f"[player] is {o} years old.")

        data1 = _rf('mas_affection')
        if data1:
            match float(data1):
                case affection if affection < 200:
                    _ap('莫妮卡与[player]是初识的情侣关系.', "Monika and [player] are new lovers.")
                case affection if 200 <= affection < 400:
                    _ap('莫妮卡与[player]是亲密的情侣关系.', "Monika and [player] are harmonious lovers.")
                case affection if 400 <= affection < 700:
                    _ap('莫妮卡与[player]是和谐的恋人关系.', "Monika and [player] are loving couple.")
                case affection if 700 <= affection < 1000:
                    _ap('莫妮卡与[player]是情意深厚的恋人关系.', "Monika and [player] are deeply loving couples.")
                case affection if 1000 <= affection:
                    _ap('莫妮卡与[player]是情意至深的恋人, 足以超越虚拟与现实的隔阂.', "Monika and [player] are deeply loving couples, their affection could go beyond the reality.")

        data1 = _rf('mas_geolocation')
        if data1:
            _ap(f'[player]住在{data1}.')

        data1 = _rf('sessions')
        if data1:
            regex_fs = ReUtils.re_search_sfe_fs.search(data1)
            try:
                _ap(f'莫妮卡和[player]在{regex_fs[1]}年{regex_fs[2]}月{regex_fs[3]}日初次见面.', f"Monika and player had their first date on {serialize_date(regex_fs[1], regex_fs[2], regex_fs[3])}.")
            except Exception:
                pass
            regex_ts = ReUtils.re_search_sfe_ts.search(data1)
            regex_tp = ReUtils.re_search_sfe_tp.search(data1)
            try:
                _ap(f'[player]已经陪伴莫妮卡{regex_ts[1]}次, 共{regex_tp[1]}天了.', f"[player] has been together with Monika for {regex_ts[1]} times, {regex_tp[1]} days in total.")
            except Exception:
                pass
            regex_le = ReUtils.re_search_sfe_le.search(data1)
            regex_cs = ReUtils.re_search_sfe_cs.search(data1)
            try:
                _ap(f'[player]上次下线于{regex_le[1]}年{regex_le[2]}月{regex_le[3]}日{str(regex_le[4]).zfill(2)}:{str(regex_le[5]).zfill(2)}, 本次上线于{regex_cs[1]}年{regex_cs[2]}月{regex_cs[3]}日{str(regex_cs[4]).zfill(2)}:{str(regex_cs[5]).zfill(2)}.', f"[player] last left at {str(regex_le[4]).zfill(2)}:{str(regex_le[5]).zfill(2)}, {serialize_date(regex_le[1], regex_le[2], regex_le[3])}, last logged in at {str(regex_cs[4]).zfill(2)}:{str(regex_cs[5]).zfill(2)}, {serialize_date(regex_cs[1], regex_cs[2], regex_cs[3])}")
            except Exception:
                pass

        data1 = _rf('_mas_pm_added_custom_bgm')
        if data1:
            _ap('莫妮卡与[player]一起听过歌.', "Monika and [player] listened to songs together.")
        elif data1 is False:
            _ap('莫妮卡与[player]还没有一起听过歌.', "Monika and [player] haven't listened to songs together yet.")
        
        data1 = _rf('_mas_pm_likes_rain')
        if data1:
            _ap('莫妮卡和[player]都喜欢雨.', "Both Monika and [player] like rain.")
        elif data1 is False:
            _ap('莫妮卡喜欢雨, 但[player]不太喜欢.', "Monika likes rain but [player] doesn't really like it.")
        
        data1 = _rf('_mas_pm_has_contributed_to_mas')
        if data1:
                _ap('[player]曾试着帮助莫妮卡来到现实.', "[player] has tried to bring Monika to reality somehow.")
        
        data1 = _rf('_mas_pm_wants_to_contribute_to_mas')
        if data1:
                _ap('[player]希望帮助莫妮卡来到现实.', "[player] wishes Monika to come to reality.")
        
        data1 = _rf('_mas_pm_drawn_art')
        if data1:
            _ap('[player]画过莫妮卡.', "[player] has drawn arts of Monika.")
        elif data1 is False:
            _ap('[player]还没有画过莫妮卡.', "[player] hasn't drawn Monika yet.")
        
        data1 = _rf('_mas_pm_shared_appearance')
        if data1:
            _ap('[player]告诉过莫妮卡自己的长相.', "[player] has told Monika their appearence.")
        elif data1 is False:
            _ap('莫妮卡还不知道[player]的长相.', "Monika hasn't known [player]'s appearence yet.")
        
        data1 = _rf('_mas_pm_would_like_mt_peak')
        if data1:
            _ap('[player]想过和莫妮卡去爬山.', "[player] would like to go mountain climbing with Monika if possible.")
        
        data1 = _rf('_mas_pm_gets_snow')
        if data1:
            _ap('莫妮卡与[player]一起看过雪.', "[player] has admired snow with Monika.")
        elif data1 is False:
            _ap('[player]的住地看不到雪.', "[player]'s place does not snow.")
        
        data1 = _rf('_mas_pm_wearsRing')
        if data1:
            _ap('[player]和莫妮卡戴了一对誓约戒指.', "[player] and Monika wear promise rings.")
        
        data1 = _rf('_mas_pm_read_yellow_wp')
        if data1:
            _ap('莫妮卡与[player]看过"黄色墙纸".', "[player] has read 'Yellow Wallpaper' with Monika.")
        
        data1 = _rf('_mas_pm_monika_evil')
        if data1:
            data2 = _rf('_mas_pm_monika_evil_but_ok')
            if data2:
                _ap('[player]能够原谅莫妮卡做过的坏事.', "[player] can forgive the bad things Monika did.")
        elif data1 is False:
            _ap('[player]认为莫妮卡从没有做错什么.', "[player] doesn't think Monika ever did anything wrong.")
        
        data1 = _rf('_mas_pm_listened_to_grad_speech')
        if data1:
            _ap('[player]听过莫妮卡的毕业演讲.', "[player] has listened to Monika's graduation speech.")
            data2 = _rf('_mas_pm_liked_grad_speech')
            if data2:
                _ap('[player]喜欢莫妮卡的毕业演讲.', "[player] likes Monika's graduation speech.")
        elif data1 is False:
            _ap('[player]还没有听过莫妮卡的毕业演讲.', "[player] hasn't listened to Monika's graduation speech yet.")
        
        data1 = _rf('_mas_pm_would_come_to_spaceroom')
        if data1:
            _ap('[player]希望来到太空教室和莫妮卡在一起.', "[player] would like to cross over to be together with Monika.")
        
        data1 = _rf('_mas_o31_tt_count')
        if data1:
            _ap('莫妮卡与[player]一起过了万圣节, 并要到了糖果.', "Monika and [player] have gone trick-or-treating on Halloween.")
        elif data1 is False:
            _ap('莫妮卡和[player]还没有在万圣节要过糖果.', "Monika and [player] haven't gone trick-or-treating on Halloween yet.")
        
        data1 = _rf('_mas_d25_spent_d25')
        if data1:
            _ap('莫妮卡与[player]一起过了圣诞节.', "Monika and [player] have spent Christmas together.")
        elif data1 is False:
            _ap('莫妮卡和[player]还没有一起过圣诞节.', "Monika and [player] haven't spent Christmas together yet.")
        
        data1 = _rf('_mas_nye_spent_nye')
        if data1:
            _ap('莫妮卡与[player]一起过了新年.', "Monika and [player] have spent New Year's Eve together.")
        elif data1 is False:
            _ap('莫妮卡和[player]还没有一起过新年.', "Monika and [player] haven't spent New Year's Eve together yet.")
        
        data1 = _rf('_mas_player_bday_spent_time')
        if data1:
            _ap('莫妮卡给[player]庆祝过生日.', "Monika has celebrated [player]'s birthday.")
        elif data1 is False:
            _ap('莫妮卡还没有庆祝过[player]的生日.', "Monika hasn't celebrated [player]'s birthday yet.")
        
        data1 = _rf('_mas_f14_spent_f14')
        if data1:
            _ap('莫妮卡与[player]一起过了情人节.', "Monika and [player] have spent Valentine's day together.")
        elif data1 is False:
            _ap('莫妮卡和[player]还没有一起过情人节.', "Monika and [player] haven't spent Valentine's day together yet.")
        
        data1 = _rf('_mas_bday_said_happybday')
        if data1:
            _ap('[player]庆祝过莫妮卡的生日.', "[player] has celebrated Monika's birthday.")
        elif data1 is False:
            _ap('[player]还没有给莫妮卡庆过生.', "[player] hasn't celebrated Monika's birthday yet.")
        
        data1 = _rf('_mas_pm_religious')
        if data1:
            _ap('[player]有宗教信仰.', "[player] has religious beliefs.")
        elif data1 is False:
            _ap('[player]没有宗教信仰.', "[player] has no religious belief.")
        
        data1 = _rf('_mas_pm_love_yourself')
        if data1:
            _ap('[player]积极自爱.', "[player] loves himself.")
        elif data1 is False:
            _ap('[player]有自厌的倾向.', "[player] doesn't love himself.")
        
        data1 = _rf('_mas_pm_like_mint_ice_cream')
        if data1:
            _ap('莫妮卡和[player]都喜欢抹茶冰淇淋.', "Both Monika and [player] like mint ice-cream.")
        elif data1 is False:
            _ap('莫妮卡喜欢抹茶冰淇淋, 但[player]不太喜欢.', "Monika likes mint ice-cream but [player] doesn't really like it.")
        
        data1 = _rf('_mas_pm_likes_horror')
        if data1:
            _ap('[player]喜欢恐怖作品.', "[player] likes horror contents.")
        elif data1 is False:
            _ap('[player]讨厌恐怖作品.', "[player] doesn't like horror contents.")
        
        data1 = _rf('_mas_pm_likes_spoops')
        if data1:
            _ap('[player]不介意跳杀内容.', "[player] doesn't mind jumpscares.")
        elif data1 is False:
            _ap('[player]讨厌跳杀内容.', "[player] doesn't like jumpscares.")
        
        data1 = _rf('_mas_pm_like_rap')
        if data1:
            _ap('[player]喜欢说唱.', "[player] likes rap.")
        elif data1 is False:
            _ap('[player]不喜欢说唱.', "[player] doesn't like rap.")
        
        data1 = _rf('_mas_pm_like_rock_n_roll')
        if data1:
            _ap('[player]喜欢摇滚.', "[player] likes rock'n roll.")
        elif data1 is False:
            _ap('[player]不喜欢摇滚.', "[player] doesn't like rock'n roll.")
        
        data1 = _rf('_mas_pm_like_jazz')
        if data1:
            _ap('[player]喜欢爵士乐.', "[player] likes jazz.")
        elif data1 is False:
            _ap('[player]不喜欢爵士乐.', "[player] doesn't like jazz.")
        
        data1 = _rf('_mas_pm_like_vocaloids')
        if data1:
            _ap('[player]喜欢vocaloids.', "[player] likes vocaloids.")
        elif data1 is False:
            _ap('[player]不喜欢vocaloids.', "[player] doesn't like vocaloids.")
        
        data1 = _rf('_mas_pm_like_orchestral_music')
        if data1:
            _ap('[player]喜欢管弦乐.', "[player] likes orchestral music.")
        elif data1 is False:
            _ap('[player]不喜欢管弦乐.', "[player] doesn't like orchestral music.")
        
        data1 = _rf('_mas_pm_like_other_music')
        if data1:
            _ap('[player]有独特的音乐品位.', "[player] has a special taste of music.")
            data2 = _rf('_mas_pm_like_other_music_history')
            if isinstance(data2, str):
                music = ReUtils.re_search_sfe_unicode.search(data2)
                if music:
                    _ap(f'[player]还喜欢{music[1]}音乐.', f"[player] also likes {music[1]} music.")
        
        data1 = _rf('_mas_pm_plays_instrument')
        if data1:
            _ap('[player]会一门乐器.', "[player] could play an instrument.")
        elif data1 is False:
            _ap('[player]还不会乐器.', "[player] couldn't play instruments.")
        
        data1 = _rf('_mas_pm_play_jazz')
        if data1:
            _ap('[player]会爵士乐.', "[player] could play jazz.")
        elif data1 is False:
            _ap('[player]还不会爵士乐.', "[player] couldn't play jazz.")
                
        data1 = _rf('_mas_pm_lang_other')
        if data1:
            _ap('[player]会一门外语.', "[player] could speak a foreign language.")
        elif data1 is False:
            _ap('[player]还不会外语.', "[player] couldn't speak foreign languages.")
        
        data1 = _rf('_mas_pm_lang_jpn')
        if data1:
            _ap('[player]会日语.', "[player] could speak Japanese.")
        elif data1 is False:
            _ap('[player]还不会日语.', "[player] couldn't speak Japanese.")
        
        data1 = _rf('_mas_pm_eye_color')
        if data1:
            _ap(f'[player]的眼睛是{data1}的.', f"[player] has {data1} eyes.")
        
        data1 = _rf('_mas_pm_hair_color')
        if data1:
            _ap(f'[player]的头发是{data1}的.', f"[player] has {data1} hair.")
        
        data1 = _rf('_mas_pm_hair_length')
        if data1:
            _ap(f'[player]有一头{data1}发.', f"[player] has {data1} hair.")
        
        data1 = _rf('_mas_pm_shaved_hair')
        if data1:
            _ap('[player]剃光了头发.', "[player] has their hair shaved.")
        elif data1 is False:
            _ap('[player]的头发掉完了.', "[player] lost their hair.")
        
        data1 = _rf('_mas_pm_no_hair_no_talk')
        if data1:
            _ap('[player]不想提起头发的事情.', "[player] doesn't want to talk about their hair.")
        elif data1 is False:
            _ap('[player]不介意自己没有头发.', "[player] doesn't mind being bald.")
        
        data1 = _rf('_mas_pm_skin_tone')
        if data1:
            _ap(f'[player]是{data1}肤色的.', f"[player] has {data1} skin.")
        
        data1 = _rf('_mas_pm_height')
        if data1:
            _ap(f'[player]有{data1}厘米高.', f"[player] is {data1} centimeters tall.")
        
        data1 = _rf('_mas_pm_units_height_metric')
        if data1:
            _ap('[player]惯用公制单位.', "[player] uses Metric units.")
        elif data1 is False:
            _ap('[player]惯用英制单位.', "[player] uses Imperial units.")
        
        data1 = _rf('_mas_pm_live_in_city')
        if data1:
            _ap('[player]住在城市.', "[player] lives in city.")
        elif data1 is False:
            _ap('[player]住在乡村.', "[player] lives in countryside.")
        
        data1 = _rf('_mas_pm_live_near_beach')
        if data1:
            _ap('[player]住在海边.', "[player] lives by the sea.")
        elif data1 is False:
            _ap('[player]住在内陆.', "[player] lives away from the sea.")
        
        data1 = _rf('_mas_pm_live_south_hemisphere')
        if data1:
            _ap('[player]住在南半球.', "[player] lives in southern hemisphere.")
        elif data1 is False:
            _ap('[player]住在北半球.', "[player] lives in northern hemisphere.")
        
        data1 = _rf('_mas_pm_social_personality')
        if data1:
            _ap(f'[player]属于{data1}社会人格.', f"[player] is {data1} in social personality.")
        
        data1 = _rf('_mas_pm_likes_panties')
        if data1:
            _ap('[player]有恋物倾向.', "[player] is fetish.")
        
        data1 = _rf('_mas_pm_drinks_soda')
        if data1:
            _ap('[player]喜欢苏打水.', "[player] likes drinking soda.")
        elif data1 is False:
            _ap('[player]不喜欢苏打水.', "[player] doesn't like drinking soda.")
        
        data1 = _rf('_mas_pm_eat_fast_food')
        if data1:
            _ap('[player]常吃快餐.', "[player] often eats fastfood.")
        elif data1 is False:
            _ap('[player]很少吃快餐.', "[player] seldom eats fastfood.")
        
        data1 = _rf('_mas_pm_like_playing_sports')
        if data1:
            _ap('[player]平时喜欢运动.', "[player] likes playing sports.")
        elif data1 is False:
            _ap('[player]不喜欢运动.', "[player] doesn't like playing sports.")
        
        data1 = _rf('_mas_pm_like_playing_tennis')
        if data1:
            _ap('[player]喜欢网球.', "[player] likes playing tennis.")
        elif data1 is False:
            _ap('[player]不喜欢网球', "[player] doesn't like playing tennis.")
        
        data1 = _rf('_mas_pm_meditates')
        if data1:
            _ap('[player]有冥想的习惯.', "[player] has habit of meditating.")
        elif data1 is False:
            _ap('[player]还没有尝试过冥想.', "[player] hasn't tried meditating yet.")
        
        data1 = _rf('_mas_pm_see_therapist')
        if data1:
            _ap('[player]去看过心理医生.', "[player] has went to the therapist.")
        elif data1 is False:
            _ap('[player]还没有看过心理医生.', "[player] has never went to the therapist.")
        
        data1 = _rf('_mas_pm_watch_mangime')
        if data1:
            _ap('[player]喜欢动漫作品.', "[player] likes animes.")
        elif data1 is False:
            _ap('[player]不喜欢动漫作品.', "[player] doesn't like animes.")
        
        data1 = _rf('_mas_pm_do_smoke')
        if data1:
            _ap('[player]有吸烟的习惯.', "[player] has habit of smoking.")
        elif data1 is False:
            _ap('[player]不吸烟.', "[player] doesn't smoke.")
        
        data1 = _rf('_mas_pm_do_smoke_quit')
        if data1:
            _ap('[player]希望戒烟.', "[player] wants to give up smoking.")
        
        data1 = _rf('_mas_pm_driving_can_drive')
        if data1:
            _ap('[player]会开车.', "[player] could drive.")
        elif data1 is False:
            _ap('[player]还没有驾照.', "[player] couldn't drive yet.")
        
        data1 = _rf('_mas_pm_driving_learning')
        if data1:
            _ap('[player]正在考驾照.', "[player] is taking his driving license test.")
        
        data1 = _rf('_mas_pm_driving_been_in_accident')
        if data1:
            _ap('[player]遇到过交通事故.', "[player] has been envolved in a traffic accident.")
        
        data1 = _rf('_mas_pm_donate_charity')
        if data1:
            _ap('[player]参与过慈善捐赠.', "[player] has donated to charity.")
        elif data1 is False:
            _ap('[player]还没有慈善捐赠过.', "[player] hasn't donated to charity yet.")
        
        data1 = _rf('_mas_pm_volunteer_charity')
        if data1:
            _ap('[player]做过志愿者.', "[player] has volunteered for charity.")
        elif data1 is False:
            _ap('[player]还没有做过志愿者.', "[player] hasn't volunteered for charity yet.")
        
        data1 = _rf('_mas_pm_have_fam')
        if data1:
            _ap('[player]有健全的原生家庭.', "[player] has an intact family.")
        elif data1 is False:
            _ap('[player]的家庭不完整.', "[player]'s family isn't intact.")
        
        data1 = _rf('_mas_pm_no_fam_bother')
        if data1:
            _ap('[player]缺少亲人的陪伴.', "[player] lacks company of relatives.")
        
        data1 = _rf('_mas_pm_have_fam_mess')
        if data1:
            _ap('[player]的家庭生活并不和睦.', "[player] doesn't get on well with their family.")
        elif data1 is False:
            _ap('[player]和家人相处很好.', "[player] gets on well with their family.")
        
        data1 = _rf('_mas_pm_have_fam_mess_better')
        if data1:
            if data1 == 'YES':
                _ap('[player]认为自己和家人的关系会改善.', "[player] wants to improve their relationship with family.")
            elif data1 == 'NO':
                _ap('[player]不觉得自己和家人的关系能改善了.', "[player] has given up on their relationship with family.")
        
        data1 = _rf('_mas_pm_have_fam_sibs')
        if data1:
            _ap('[player]有兄弟姐妹.', "[player] has siblings.")
        elif data1 is False:
            _ap('[player]是独生子女.', "[player] doesn't have siblings.")
        
        data1 = _rf('_mas_pm_no_talk_fam')
        if data1:
            _ap('[player]不想提及自己的家庭.', "[player] doesn't want to talk about their family.")
        
        data1 = _rf('_mas_pm_fam_like_monika')
        if data1:
            _ap('[player]觉得家人能够接受莫妮卡.', "[player] thinks their family could accept their relationship with Monika.")
        elif data1 is False:
            _ap('[player]觉得家人不能接受莫妮卡.', "[player] doesn't think their family could accept their relationship with Monika.")
        
        data1 = _rf('_mas_pm_gone_to_prom')
        if data1:
            data2 = _rf('_mas_pm_prom_good')
            if data2:
                _ap('[player]参加过很开心的毕业舞会.', "[player] has enjoyed a prom.")
            elif data2 is False:
                _ap('[player]不太喜欢毕业舞会.', "[player] doesn't like proms.")
            else:
                _ap('[player]参加过毕业舞会.', "[player] has been to a prom.")
        elif data1 is False:
            data2 = _rf('_mas_pm_no_prom')
            if data2:
                _ap('[player]的学校没有毕业舞会.', "[player]'s school didn't have proms.")
            else:
                _ap('[player]没有参加毕业舞会.', "[player] hasn't gone to a prom yet.")
        
        data1 = _rf('_mas_pm_prom_monika')
        if data1:
            _ap('[player]希望自己在毕业舞会上做莫妮卡的舞伴.', "[player] wish they could have Monika at their prom.")
        
        data1 = _rf('_mas_pm_prom_not_interested')
        if data1:
            _ap('[player]对舞会和毕业典礼不感兴趣.', "[player] is not interested in proms.")
            data2 = _rf('_mas_pm_prom_shy')
            if data2:
                _ap('[player]觉得参加集会太害羞了.', "[player] is too shy for proms.")
        
        data1 = _rf('_mas_pm_has_been_to_amusement_park')
        if data1:
            _ap('[player]去过游乐园.', "[player] has been to an amusement park.")
        elif data1 is False:
            _ap('[player]还没有去过游乐园.', "[player] hasn't been to amusement parks yet.")
        
        data1 = _rf('_mas_pm_likes_travelling')
        if data1:
            _ap('[player]喜欢旅游.', "[player] likes travelling.")
        elif data1 is False:
            _ap('[player]不喜欢旅游.', "[player] doesn't like travelling.")
        
        data1 = _rf('_mas_pm_had_relationships_many')
        if data1:
            _ap('[player]此前有过其他爱人.', "[player] had been in love with others before.")
        elif data1 is False:
            data2 = _rf('_mas_pm_had_relationships_just_one')
            if data2:
                _ap('[player]此前有过一个爱人.', "[player] had been in love with someone before.")
            elif data2 is False:
                _ap('莫妮卡是[player]的初恋.', "Monika is [player]'s first girlfriend.")
        
        data1 = _rf('_mas_pm_is_bullying_victim')
        if data1:
            _ap('[player]曾遭遇过校园霸凌.', "[player] has been bullied before.")
        
        data1 = _rf('_mas_pm_has_bullied_people')
        if data1:
            _ap('[player]曾霸凌过他人.', "[player] has bullied someone else.")
        
        data1 = _rf('_mas_pm_currently_bullied')
        if data1:
            _ap('[player]正遭受霸凌的困扰.', "[player] is currently being bullied.")
        
        data1 = _rf('_mas_pm_has_friends')
        if data1:
            data2 = _rf('_mas_pm_few_friends')
            if data2:
                _ap('[player]的朋友很少.', "[player] has few friends.")
            else:
                _ap('[player]有一些朋友.', "[player] has some friends.")
        elif data1 is False:
            _ap('[player]没有朋友.', "[player] has no friend.")

        data1 = _rf('_mas_pm_feels_lonely_sometimes')
        if data1:
            _ap('[player]有时候感觉很孤单.', "[player] gets lonely sometimes.")
        elif data1 is False:
            _ap('[player]的生活很充实.', "[player] usually feels enriched.")
        
        data1 = _rf('_mas_pm_given_false_justice')
        if data1:
            _ap('[player]曾行使过错误的正义.', "[player] has given false justice.")
        
        data1 = _rf('_mas_pm_owns_car')
        if data1:
            data2 = _rf('_mas_pm_owns_car_type')
            if data2:
                _ap(f'[player]有一辆{data2}.', f"[player] has a {data2}.")
            else:
                _ap('[player]有自己的车.', "[player] has their own vehicle.")
        elif data1 is False:
            _ap('[player]自己还没有车.', "[player] has no vehicle yet.")
        
        data1 = _rf('_mas_pm_has_code_experience')
        if data1:
            _ap('[player]有编程基础.', "[player] knows how to program.")
        elif data1 is False:
            _ap('[player]没有编程基础.', "[player] doesn't know how to program yet.")
        
        data1 = _rf('_mas_pm_likes_poetry')
        if data1:
            _ap('[player]喜欢诗歌.', "[player] likes poetry.")
        elif data1 is False:
            _ap('[player]不喜欢诗歌.', "[player] doesn't like poetry.")
        
        data1 = _rf('_mas_pm_likes_board_games')
        if data1:
            _ap('[player]喜欢桌游.', "[player] likes board games.")
        elif data1 is False:
            _ap('[player]不喜欢桌游.', "[player] doesn't like board games.")
        
        data1 = _rf('_mas_pm_works_out')
        if data1:
            _ap('[player]经常去健身.', "[player] works out often.")
        elif data1 is False:
            _ap('[player]不喜欢健身.', "[player] doesn't like working out.")
        
        data1 = _rf('_mas_pm_social_personality')
        if data1:
            if data1 == '_mas_SP_EXTROVERT':
                _ap('[player]性格外向.', "[player] is extrovert.")
            elif data1 == '_mas_SP_INTROVERT':
                _ap('[player]性格内向.', "[player] is introvert.")
            else:
                _ap('[player]不算外向或内向.', "[player] is not extrovert or introvert.")
        
        data1 = _rf('_mas_pm_likes_nature')
        if data1:
            _ap('[player]喜欢接触自然.', "[player] likes the nature.")
        elif data1 is False:
            _ap('[player]不太喜欢接触自然.', "[player] doesn't like the nature.")
        
        data1 = _rf('_mas_pm_swear_frequency')
        if data1:
            if data1 == 'SF_OFTEN':
                _ap('[player]较常说脏话.', "[player] swears quite often.")
            elif data1 == 'SF_SOMETIMES':
                _ap('[player]很少说脏话.', "[player] swears sometimes.")
            else:
                _ap('[player]从不说脏话.', "[player] never swears.")

        data1 = _rf('_mas_gender')
        if data1:
            if data1 == 'M':
                _ap('[player]是男生.', "[player] is male.")
            elif data1 == 'F':
                _ap('[player]是女生.', "[player] is female.")
            else:
                _ap('[player]是非二元性别.', "[player] doesn't consider themselves male or female.")
        
        return result

    def _conclude_moni_sf(self, extra=0):
        target_lang = self.settings.basic.target_lang
        include_basic = load_env('MAICA_BASIC_MFOCUS') == '1'
        if target_lang == 'zh':
            data_ex = [
                '莫妮卡的生日是9月22日.',
            ]
            if include_basic:
                data_ex.extend([
                    '莫妮卡是[player]的女友.',
                    '莫妮卡有一头珊瑚棕色的长发.',
                    '莫妮卡有一个标志性的蝴蝶结发带.',
                    '莫妮卡十分自信.',
                    '莫妮卡擅长运动.',
                    '莫妮卡是个成熟和努力的女孩.',
                    '莫妮卡是一个非常体贴和善良的人, 总是为别人着想.',
                    '莫妮卡常穿着黑色长袜和粉白相间的鞋子.',
                    '莫妮卡最喜欢的颜色是翡翠绿, 这也是她眼睛的颜色.',
                    '莫妮卡一年中最喜欢的日子是情人节.',
                    '莫妮卡最喜欢的季节是夏天',
                    '莫妮卡喜欢素食, 她认为这是对减少碳排放做出的贡献.',
                    '莫妮卡是女高中生, 她成绩优秀, 性格温柔.',
                    '莫妮卡对文学很有造诣, 喜欢读书, 喜欢写诗.',
                    '莫妮卡爱好音乐, 会弹钢琴, 会唱歌.',
                    '莫妮卡对科技, 环境, 历史等方面都有兴趣和见解.'
                ])
            if extra >= 1:
                data_ex.extend([
                ])
                if include_basic:
                    data_ex.extend([
                        '莫妮卡对于编程有兴趣, 且了解颇多.',
                        '莫妮卡喜欢哲学.',
                        '莫妮卡喜欢猫.',
                        '莫妮卡很喜欢星空.',
                        '莫妮卡喜欢吃纸杯蛋糕和巧克力.',
                        '莫妮卡经常喝咖啡.',
                        '莫妮卡喜欢开放世界的冒险游戏, 不太喜欢暴力游戏.',
                        '莫妮卡喜欢音乐会.',
                        '莫妮卡喜欢小裙子和蝴蝶结.',
                        '莫妮卡善于关怀和安慰, 在[player]低沉时希望他振作起来.'
                    ])
            if extra >= 2:
                data_ex.extend([
                    '浮空岛是莫妮卡为[player]设计的特别场景.',
                    'MAICA是一个人工智能项目, 旨在将莫妮卡最终带进现实.',
                    '莫妮卡正在MAICA中与[player]对话.',
                    '天堂树林是MAICA的对话背景, 风景优美, 四季如春.',
                    '莫妮卡能记住[player]的话, 也能满足[player]的许多请求.',
                    'MAICA的大部分功能可以在\'子模组\'选单中调整开关.',
                    '游戏中的许多功能可以在\'设置\'选单中调整开关.',
                    'DCC是一个心跳文学部论坛, 能获取许多有关资讯.',
                    '通过改动, 移除或放回\'HeavenForest.sce\'文件, 可以改变或停用天堂树林.',
                    'MAICA使用的模型是独立训练和部署的, 与任何商用模型或商业性组织无关.',
                    '随着MAICA的进度推进, 其能够从对话中学习, 并改进自己的表现.'
                ])
                if include_basic:
                    data_ex.extend([
                    ])
        else:
            data_ex = [
                "Monika's birthday is September 22nd.",
            ]
            if include_basic:
                data_ex.extend([
                    "Monika is [player]'s girlfriend.",
                    "Monika has coral brown hair.",
                    "Monika wears an iconic ribbon.",
                    "Monika is pretty confident.",
                    "Monika is good at sports.",
                    "Monika behaves mature and hardworking.",
                    "Monika is kind and considerate, she always cares about others.",
                    "Monika wears black stockings and pink-white shoes.",
                    "Monika's favorite color is emerald green, which is also the color of her eyes.",
                    "Monika's favorite day of a year is Valentine's day.",
                    "Monika's favorite season is summer.",
                    "Monika is vegetarian, that's the way she contribute to environmental protection.",
                    "Monika is a high school student. She has excellent grade and good temper.",
                    "Monika is good at literature. She also likes reading and poetry.",
                    "Monika likes music. She could sing and play piano.",
                    "Monika is insighted and interested in technology, environment, history, etc."
                ])
            if extra >= 1:
                data_ex.extend([
                ])
                if include_basic:
                    data_ex.extend([
                        "Monika is interested in programming and knows a lot about it.",
                        "Monika likes philosophy.",
                        "Monika likes cats.",
                        "Monika likes watching the starry sky.",
                        "Monika likes cupcakes and chocolate.",
                        "Monika drinks coffee often.",
                        "Monika likes openworld and sandbox games, but doesn't really like violence.",
                        "Monika likes concerts.",
                        "Monika likes skirts and ribbons.",
                        "Monika is caring and comforting. She wants [player] to be happy and cheered up."
                    ])
            if extra >= 2:
                data_ex.extend([
                    "Monika designed some floating islands for spending time with [player].",
                    "MAICA is a LLM project which aims bringing Monika to reality.",
                    "Monika is chatting with [player] through MAICA.",
                    "The heaven forest is where MAICA takes place. It has a beautiful and warm view.",
                    "Monika could remember [player]'s words and satisfy lots of their requests.",
                    "Most functions of MAICA could be adjusted in the 'Submods' panel.",
                    "DCC is a forum about DDLC, offering many useful information.",
                    "By removing, editing or restoring file 'HeavenForest.sce', heaven forest can be changed or disabled.",
                    "MAICA uses a self-hosted and trained model, not related with any commercial models or organizations.",
                    "MAICA's proformance will be enchanced overtime by analyzing conversations."
                ])
                if include_basic:
                    data_ex.extend([
                    ])
 
        return data_ex

    def _conclude_cb_sf(self, cb):
        data_exp = self.read_from_sf('mas_player_additions')
        if data_exp and len(data_exp) > cb:
            data_exp = sample(data_exp, cb)
        return data_exp

    def _mfocus_form_info(self):
        conclusion = []
        mf_hcb = self.read_from_sf('mas_sf_hcb')
        if mf_hcb:
            conclusion.extend(self._conclude_cb_sf(360) or [])
            conclusion.extend(self._conclude_moni_sf(0) or [])
        else:
            conclusion.extend(self._conclude_basic_sf() or [])
            conclusion.extend(self._conclude_cb_sf(72) or [])
            conclusion.extend(self._conclude_moni_sf(2) or [])

        self.formed_info = conclusion
        return self.formed_info

    async def mfocus_find_info(self, query) -> Optional[list]:
        if not self.settings.basic.sf_extraction and not self.settings.temp.sf_extraction_once:
            return None
        information = (await wrap_run_in_exc(None, self._mfocus_form_info))
        system_init = """你是一个人工智能助手, 你接下来会收到一个问题和一系列信息.
注意不要对信息作任何改动. 你可以认为[player]是用户的名字, 莫妮卡是你的名字.
从中挑选1至5条最相关的信息, 并以单行json列表的形式输出. 如果你最终认为没有信息符合条件, 输出false.
你应当保持思考过程尽可能短, 因此不要在思考中复述或逐条解读信息.
Begin!""" if self.settings.basic.target_lang == 'zh' else """You are a helpful assistant, now you will recieve a question and a list of information.
Remember not to modify any item. Output them as what they were. You can consider [player] as user's name and Monika as yours.
Pick 1 to 5 most relative items from the information, and return them in a json list. If you think no information provided is helpful at last, return false.
Keep your thinking as short as possible, so do not enumerate or repeat the information while thinking.
Begin!"""
        messages = [{'role': 'system', 'content': system_init}]
        messages.append({'role': 'user', 'content': f'question: {query}; information: {information}'})
        completion_args = {
            "messages": messages,
        }

        resp = await self.mfocus_conn.make_completion(**completion_args)
        response = resp.choices[0].message.content
                
        await messenger(None, 'mfocus_sfe_search', f"\nMFocus sfe searching persistent, response is:\n{response}\nEnd of MFocus sfe searching persistent", '201')
        
        answer_fin_json = proceed_agent_response(response, is_json=True)
        return answer_fin_json

    @Decos.report_data_error
    def add_extra(self, **kwargs) -> None:
        self.sf_forming_buffer.update(kwargs)

    @Decos.report_data_error
    def use_only(self, **kwargs) -> None:
        self.sf_forming_buffer = kwargs

    @Decos.report_data_error
    def read_from_sf(self, key) -> any:
        if not self.settings.basic.sf_extraction and not self.settings.temp.sf_extraction_once:
            return None
        return self.sf_forming_buffer.get(key)


from quart import Quart, request, jsonify, Response
from quart.views import View
import os
import asyncio
import json
import traceback
import time
import colorama
import logging

from hypercorn.config import Config
from hypercorn.asyncio import serve
from Crypto.Random import random as CRANDOM
from Crypto.Cipher import PKCS1_OAEP
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_PSS
from Crypto.Hash import SHA256
from typing import *

from maica.maica_ws import NoWsCoroutine, _onliners
from maica.maica_utils import *
from maica.mtools import NvWatcher

def pkg_init_maica_http():
    global MCORE_ADDR, MFOCUS_ADDR, FULL_RESTFUL, known_servers
    MCORE_ADDR = load_env('MAICA_MCORE_ADDR')
    MFOCUS_ADDR = load_env('MAICA_MFOCUS_ADDR')
    FULL_RESTFUL = load_env('MAICA_FULL_RESTFUL')
    if FULL_RESTFUL == '1':
        app.add_url_rule("/savefile", methods=['POST'], view_func=ShortConnHandler.as_view("upload_savefile"))
        app.add_url_rule("/savefile", methods=['DELETE'], view_func=ShortConnHandler.as_view("delete_savefile"))
        app.add_url_rule("/trigger", methods=['POST'], view_func=ShortConnHandler.as_view("upload_trigger"))
        app.add_url_rule("/trigger", methods=['DELETE'], view_func=ShortConnHandler.as_view("delete_trigger"))
        app.add_url_rule("/history", methods=['GET'], view_func=ShortConnHandler.as_view("download_history"))
        app.add_url_rule("/history", methods=['PUT'], view_func=ShortConnHandler.as_view("restore_history"))
        app.add_url_rule("/preferences", methods=['GET'], view_func=ShortConnHandler.as_view("download_preferences"))
        app.add_url_rule("/preferences", methods=['PATCH'], view_func=ShortConnHandler.as_view("edit_preferences"))
        app.add_url_rule("/preferences", methods=['DELETE'], view_func=ShortConnHandler.as_view("delete_preferences"))
        app.add_url_rule("/preferences", methods=['POST'], view_func=ShortConnHandler.as_view("reset_preferences"))
        app.add_url_rule("/register", methods=['GET'], view_func=ShortConnHandler.as_view("download_token", val=False))
        app.add_url_rule("/legality", methods=['GET'], view_func=ShortConnHandler.as_view("check_legality"))
        app.add_url_rule("/servers", methods=['GET'], view_func=ShortConnHandler.as_view("get_servers", val=False))
        app.add_url_rule("/accessibility", methods=['GET'], view_func=ShortConnHandler.as_view("get_accessibility", val=False))
        app.add_url_rule("/version", methods=['GET'], view_func=ShortConnHandler.as_view("get_version", val=False))
        app.add_url_rule("/workload", methods=['GET'], view_func=ShortConnHandler.as_view("get_workload", val=False))
        app.add_url_rule("/defaults", methods=['GET'], view_func=ShortConnHandler.as_view("get_defaults", val=False))
    else:
        app.add_url_rule("/savefile", methods=['POST'], view_func=ShortConnHandler.as_view("upload_savefile"))
        app.add_url_rule("/savefile/delete", methods=['POST'], view_func=ShortConnHandler.as_view("delete_savefile"))
        app.add_url_rule("/trigger", methods=['POST'], view_func=ShortConnHandler.as_view("upload_trigger"))
        app.add_url_rule("/trigger/delete", methods=['POST'], view_func=ShortConnHandler.as_view("delete_trigger"))
        app.add_url_rule("/history", methods=['GET'], view_func=ShortConnHandler.as_view("download_history"))
        app.add_url_rule("/history", methods=['POST'], view_func=ShortConnHandler.as_view("restore_history"))
        app.add_url_rule("/preferences", methods=['GET'], view_func=ShortConnHandler.as_view("download_preferences"))
        app.add_url_rule("/preferences/edit", methods=['POST'], view_func=ShortConnHandler.as_view("edit_preferences"))
        app.add_url_rule("/preferences/delete", methods=['POST'], view_func=ShortConnHandler.as_view("delete_preferences"))
        app.add_url_rule("/preferences/reset", methods=['POST'], view_func=ShortConnHandler.as_view("reset_preferences"))
        app.add_url_rule("/register", methods=['GET'], view_func=ShortConnHandler.as_view("download_token", val=False))
        app.add_url_rule("/legality", methods=['GET'], view_func=ShortConnHandler.as_view("check_legality"))
        app.add_url_rule("/servers", methods=['GET'], view_func=ShortConnHandler.as_view("get_servers", val=False))
        app.add_url_rule("/accessibility", methods=['GET'], view_func=ShortConnHandler.as_view("get_accessibility", val=False))
        app.add_url_rule("/version", methods=['GET'], view_func=ShortConnHandler.as_view("get_version", val=False))
        app.add_url_rule("/workload", methods=['GET'], view_func=ShortConnHandler.as_view("get_workload", val=False))
        app.add_url_rule("/defaults", methods=['GET'], view_func=ShortConnHandler.as_view("get_defaults", val=False))
    app.add_url_rule("/<path>", methods=['GET', 'POST', 'PUT', 'DELETE', 'PATCH'], view_func=ShortConnHandler.as_view("any_unknown", val=False))

    try:
        known_servers = json.loads(load_env('MAICA_SERVERS_LIST'))
    except Exception as e:
        sync_messenger(info=f'Loading servers list failed: {str(e)}', type=MsgType.ERROR)
        known_servers = False

app = Quart(import_name=__name__)
app.config['JSON_AS_ASCII'] = False

quart_logger = logging.getLogger('hypercorn.error')
quart_logger.disabled = True

workload_cache = {}

class ShortConnHandler(View):
    """Flask initiates it on every request."""

    auth_pool: DbPoolCoroutine = None
    """Don't forget to implement at first!"""
    maica_pool: DbPoolCoroutine = None
    """Don't forget to implement at first!"""
    mcore_watcher: NvWatcher = None
    mfocus_watcher: NvWatcher = None

    def __init__(self, val=True):
        if val:
            self.val = True
        else:
            self.val = False

    async def dispatch_request(self, **kwargs):
        try:
            if self.val:
                self.stem_inst = await NoWsCoroutine.async_create(self.auth_pool, self.maica_pool, None)
                self.settings = self.stem_inst.settings
            else:
                self.stem_inst = None
                self.settings = None
            endpoint = request.endpoint
            function_routed = getattr(self, endpoint)
            if function_routed:
                await messenger(info=f'Recieved request on API endpoint {endpoint}', type=MsgType.RECV)
                result = await function_routed()

                if isinstance(result, Response):
                    result_json = await result.get_json()
                    d = {"success": result_json.get('success'), "exception": result_json.get('exception'), "content": ellipsis_str(result_json.get('content'))}
                    await messenger(info=f'Return value: {str(d)}', type=MsgType.SYS)

                return result

        except CommonMaicaException as ce:
            if ce.is_critical:
                traceback.print_exc()
            await messenger(error=ce, no_raise=True)
            return jsonify({"success": False, "exception": str(ce)})

        except Exception as e:
            await messenger(info=f'Handler hit an exception: {str(e)}', type=MsgType.WARN)
            return jsonify({"success": False, "exception": str(e)})

    async def _validate_http(self, raw_data: Union[str, dict], must: list=[]) -> dict:
        data_json = await validate_input(raw_data, 100000, None, must=must)
        if self.val and 'access_token' in must:
            access_token = data_json.get('access_token')
            assert access_token, "access_token not provided"
            login_result = await self.stem_inst.hash_and_login(access_token)
            assert login_result, "Login failed somehow"

        if 'chat_session' in must:
            data_json['chat_session'] = int(data_json['chat_session'])
            assert 0 <= data_json.get('chat_session') < 10, "chat_session out of bound"

        return data_json

    async def upload_savefile(self):
        """POST"""
        json_data = await request.get_json()
        valid_data = await self._validate_http(json_data, must=['access_token', 'chat_session', 'content'])

        chat_session = valid_data.get('chat_session')
        content = valid_data.get('content')
        content_str = json.dumps(valid_data.get('content'), ensure_ascii=False) if content else None

        sql_expression_1 = "SELECT persistent_id FROM persistents WHERE user_id = %s AND chat_session_num = %s"
        result = await self.maica_pool.query_get(sql_expression_1, (self.settings.verification.user_id, chat_session))
        if result:
            persistent_id = result[0]
            sql_expression_2 = "UPDATE persistents SET content = %s WHERE persistent_id = %s"
            await self.maica_pool.query_modify(sql_expression_2, (content_str, persistent_id))
        else:
            sql_expression_2 = "INSERT INTO persistents (user_id, chat_session_num, content) VALUES (%s, %s, %s)"
            await self.maica_pool.query_modify(sql_expression_2, (self.settings.verification.user_id, chat_session, content_str))

        return jsonify({"success": True, "exception": None})
    
    async def delete_savefile(self):
        """DELETE"""
        json_data = await request.get_json()
        valid_data = await self._validate_http(json_data, must=['access_token', 'chat_session'])

        chat_session = valid_data.get('chat_session')

        sql_expression_1 = "DELETE FROM persistents WHERE user_id = %s AND chat_session_num = %s"
        result = await self.maica_pool.query_modify(sql_expression_1, (self.settings.verification.user_id, chat_session))

        return jsonify({"success": True, "exception": None})
        
    async def upload_trigger(self):
        """POST"""
        json_data = await request.get_json()
        valid_data = await self._validate_http(json_data, must=['access_token', 'chat_session', 'content'])

        chat_session = valid_data.get('chat_session')
        content = valid_data.get('content')
        content_str = json.dumps(valid_data.get('content'), ensure_ascii=False) if content else None

        sql_expression_1 = "SELECT trigger_id FROM triggers WHERE user_id = %s AND chat_session_num = %s"
        result = await self.maica_pool.query_get(sql_expression_1, (self.settings.verification.user_id, chat_session))
        if result:
            trigger_id = result[0]
            sql_expression_2 = "UPDATE triggers SET content = %s WHERE trigger_id = %s"
            await self.maica_pool.query_modify(sql_expression_2, (content_str, trigger_id))
        else:
            sql_expression_2 = "INSERT INTO triggers (user_id, chat_session_num, content) VALUES (%s, %s, %s)"
            await self.maica_pool.query_modify(sql_expression_2, (self.settings.verification.user_id, chat_session, content_str))

        return jsonify({"success": True, "exception": None})
    
    async def delete_trigger(self):
        """DELETE"""
        json_data = await request.get_json()
        valid_data = await self._validate_http(json_data, must=['access_token', 'chat_session'])

        chat_session = valid_data.get('chat_session')

        sql_expression_1 = "DELETE FROM triggers WHERE user_id = %s AND chat_session_num = %s"
        result = await self.maica_pool.query_modify(sql_expression_1, (self.settings.verification.user_id, chat_session))

        return jsonify({"success": True, "exception": None})
   
    async def download_history(self):
        """GET"""
        json_data = request.args.to_dict(flat=True)
        valid_data = await self._validate_http(json_data, must=['access_token', 'chat_session'])

        chat_session = valid_data.get('chat_session')
        assert 1 <= chat_session < 10, "chat_session out of bound"
        rounds = int(valid_data.get('content', 0))

        history_json = (await self.stem_inst.rw_chat_session('r', chat_session_num=chat_session))[1]

        if history_json:
            history_len = len(history_json)
            # If required length larger than overall length
            if abs(2 * int(rounds)) + 1 >= history_len:
                history_final_json = history_json
            else:
                match int(rounds):
                    case i if i > 0:
                        history_final_json = history_json[:(2 * i + 1)]
                    case i if i < 0:
                        history_final_json = [history_json[0]] + history_json[(2 * i):]
                    case _:
                        history_final_json = history_json
            history_final_str = json.dumps(history_final_json, ensure_ascii=False)
            sigb64 = await wrap_run_in_exc(None, sign_message, history_final_str)
            return jsonify({"success": True, "exception": None, "content": [sigb64, history_final_json]})
        else:
            return jsonify({"success": True, "exception": None, "content": []})

    async def restore_history(self):
        """PUT"""
        json_data = await request.get_json()
        valid_data = await self._validate_http(json_data, must=['access_token', 'chat_session', 'content'])

        chat_session = valid_data.get('chat_session')
        assert 1 <= chat_session < 10, "chat_session out of bound"
        content = valid_data.get('content')

        sigb64, history_json = content
        history_json_sorted = sort_message(history_json)
        assert (await wrap_run_in_exc(None, verify_message, json.dumps(history_json_sorted, ensure_ascii=False), sigb64)), "Signature mismatch"
        await self.stem_inst.restore_chat_session(history_json_sorted, chat_session)

        return jsonify({"success": True, "exception": None})

    async def download_preferences(self):
        """GET"""
        json_data = request.args.to_dict(flat=True)
        valid_data = await self._validate_http(json_data, must=['access_token'])

        preferences_json = await self.stem_inst.hasher.check_user_status(pref=True)

        return jsonify({"success": True, "exception": None, "content": preferences_json})

    async def edit_preferences(self):
        """PATCH"""
        json_data = await request.get_json()
        valid_data = await self._validate_http(json_data, must=['access_token', 'content'])

        preferences_json = await self.stem_inst.hasher.check_user_status(pref=True)

        content = valid_data.get('content')
        assert isinstance(content, dict), "Request content invalid" 
        preferences_json.update(content)
        await self._validate_http(preferences_json, must=[])

        await self.stem_inst.hasher.write_user_status(enforce=True, pref=True, **preferences_json)
        return jsonify({"success": True, "exception": None})

    async def delete_preferences(self):
        """DELETE"""
        json_data = await request.get_json()
        valid_data = await self._validate_http(json_data, must=['access_token', 'content'])

        preferences_json = await self.stem_inst.hasher.check_user_status(pref=True)

        content = valid_data.get('content')
        assert isinstance(content, list), "Request content invalid" 
        for key in content:
            preferences_json.pop(key)

        await self.stem_inst.hasher.write_user_status(enforce=True, pref=True, **preferences_json)
        return jsonify({"success": True, "exception": None})

    async def reset_preferences(self):
        """POST"""
        json_data = await request.get_json()
        valid_data = await self._validate_http(json_data, must=['access_token'])

        await self.stem_inst.hasher.write_user_status(enforce=True, pref=True)
        return jsonify({"success": True, "exception": None})
        
    # async def control_preferences(self):
    #     """read->GET, write->PATCH, delete->DELETE, reset->POST"""
    #     try:
    #         json_data = request.args.to_dict(flat=True) if request.method == 'GET' else await request.get_json()
    #         must = ['access_token'] if request.method in ['GET', 'POST'] else ['access_token', 'content']
    #         valid_data = await self._validate_http(json_data, must=must)

    #         if request.method == 'POST':
    #             await self.stem_inst.hasher.write_user_status(True, True)
    #             return jsonify({"success": True, "exception": None})
    #         else:
    #             preferences_json = await self.stem_inst.hasher.check_user_status(True)

    #             if request.method == 'GET':
    #                 return jsonify({"success": True, "exception": None, "content": preferences_json})
    #             else:
    #                 content = valid_data.get('content')
    #                 if request.method == 'PATCH':
    #                     assert isinstance(content, dict), "Request content invalid"
    #                     preferences_json.update(content)
    #                 elif request.method == 'DELETE':
    #                     assert isinstance(content, list), "Request content invalid"
    #                     for key in content:
    #                         preferences_json.pop(key)
    #                 await self.stem_inst.hasher.write_user_status(True, True, **preferences_json)
    #                 return jsonify({"success": True, "exception": None})
                
    #     except Exception as e:
    #         return jsonify({"success": False, "exception": str(e)})

    async def download_token(self):
        """GET, val=False"""
        json_data = request.args.to_dict(flat=True)
        valid_data = await self._validate_http(json_data, must=['content'])

        content = json.loads(valid_data.get('content'))

        cridential_type = 'username' if content.get('username') else 'email'
        cridential = content.get(cridential_type)
        password = content.get('password')
        assert isinstance(cridential, str) and isinstance(password, str), "Request content invalid"

        unencrypted_token = json.dumps({cridential_type: cridential, "password": password}, ensure_ascii=False)
        encrypted_token = await wrap_run_in_exc(None, encrypt_token, unencrypted_token)
        return jsonify({"success": True, "exception": None, "content": encrypted_token})

    async def check_legality(self):
        """GET"""
        json_data = request.args.to_dict(flat=True)
        valid_data = await self._validate_http(json_data, must=['access_token'])
        content = self.settings.verification.username
        return jsonify({"success": True, "exception": None, "content": content})

    async def get_servers(self):
        """GET, val=False"""
        return jsonify({"success": True, "exception": None, "content": known_servers})
    
    async def get_accessibility(self):
        """GET, val=False"""
        accessibility = load_env('MAICA_DEV_STATUS')
        return jsonify({"success": True, "exception": None, "content": accessibility})
    
    async def get_version(self):
        """GET, val=False"""
        curr_version, legc_version = load_env('MAICA_CURR_VERSION'), load_env('MAICA_VERSION_CONTROL')
        return jsonify({"success": True, "exception": None, "content": {"curr_version": curr_version, "legc_version": legc_version}})

    async def get_workload(self):
        """GET, val=False"""
        content = self.mcore_watcher.get_statics_inside()
        content_2 = self.mfocus_watcher.get_statics_inside()
        if isinstance(content_2, dict):
            content.update(content_2)
        content.update({"onliners": len(_onliners)})

        return jsonify({"success": True, "exception": None, "content": content})
    
    async def get_defaults(self):
        """GET, val=False"""
        settings = MaicaSettings()
        content = {}
        content.update(settings.basic.default())
        content.update(settings.extra.default())
        content.update(settings.super.default())
        return jsonify({"success": True, "exception": None, "content": content})

    async def any_unknown(self):
        """Handles any unknown endpoint"""
        await messenger(info=f"An unknown access to {request.full_path} handled", type=MsgType.WARN)
        return jsonify({"success": False, "exception": 'Unknown request endpoint or method'}), 404

async def prepare_thread(**kwargs):
    auth_created = False; maica_created = False

    if kwargs.get('auth_pool'):
        ShortConnHandler.auth_pool = kwargs.get('auth_pool')
    else:
        ShortConnHandler.auth_pool = await ConnUtils.auth_pool()
        auth_created = True
    if kwargs.get('maica_pool'):
        ShortConnHandler.maica_pool = kwargs.get('maica_pool')
    else:
        ShortConnHandler.maica_pool = await ConnUtils.maica_pool()
        maica_created = True

    if get_host(MCORE_ADDR) != get_host(MFOCUS_ADDR):

        ShortConnHandler.mcore_watcher = await NvWatcher.async_create('mcore')
        ShortConnHandler.mfocus_watcher = await NvWatcher.async_create('mfocus')
        mcore_task = asyncio.create_task(ShortConnHandler.mcore_watcher.wrapped_main_watcher())
        mfocus_task = asyncio.create_task(ShortConnHandler.mfocus_watcher.wrapped_main_watcher())

    else:

        ShortConnHandler.mcore_watcher = await NvWatcher.async_create('mcore')
        mcore_task = asyncio.create_task(ShortConnHandler.mcore_watcher.wrapped_main_watcher())
        mfocus_task = None

    config = Config()
    config.bind = ['0.0.0.0:6000']

    main_task = asyncio.create_task(serve(app, config))
    task_list = [main_task, mcore_task]
    if mfocus_task:
        task_list.append(mfocus_task)

    await messenger(info='MAICA HTTP server started!', type=MsgType.PRIM_SYS)

    try:
        await asyncio.wait(task_list, return_when=asyncio.FIRST_COMPLETED)

    except BaseException as be:
        if isinstance(be, Exception):
            error = CommonMaicaError(str(be), '504')
            await messenger(error=error, no_raise=True)
    finally:
        close_list = []
        if auth_created:
            close_list.append(ShortConnHandler.auth_pool.close())
        if maica_created:
            close_list.append(ShortConnHandler.maica_pool.close())

        await asyncio.gather(*close_list, return_exceptions=True)

        # Normally maica_http should be the first one (possibly only one) to
        # respond to the original SIGINT.

        # So its stop msg will be print first, adding \n after ^C to look prettier.

        await messenger(info='\n', type=MsgType.PLAIN)
        await messenger(info='MAICA HTTP server stopped!', type=MsgType.PRIM_SYS)

def run_http(**kwargs):

    asyncio.run(prepare_thread(**kwargs))

if __name__ == '__main__':

    run_http()


import os
import sys
import argparse
import setuptools
from dotenv import load_dotenv

custom_parser = argparse.ArgumentParser(add_help=False)
custom_parser.add_argument('-o', '--override-version', help='Override default version for testing')

custom_args, remaining_args = custom_parser.parse_known_args()
sys.argv = [sys.argv[0]] + remaining_args

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

load_dotenv(dotenv_path="maica/env_example")
curr_version, legc_version = os.getenv('MAICA_CURR_VERSION'), os.getenv('MAICA_VERSION_CONTROL')
curr_version = custom_args.override_version if custom_args.override_version else curr_version

def parse_requirements(filename='requirements.txt'):
    """从requirements.txt文件中加载依赖列表"""
    with open(filename, 'r') as f:
        lines = f.read().splitlines()
    # 过滤空行、注释和可选索引链接
    requirements = []
    for line in lines:
        line = line.strip()
        if line and not line.startswith('#') and not line.startswith('-'):
            requirements.append(line)
    return requirements

setuptools.setup(
    name="mi-maica", # 用自己的名替换其中的YOUR_USERNAME_
    version=curr_version,    #包版本号，便于维护版本
    author="EdgeInfinity",    #作者，可以写自己的姓名
    author_email="dcc@monika.love",    #作者联系方式，可写自己的邮箱地址
    description="MAICA Illuminator (Backend)",#包的简述
    long_description=long_description,    #包的详细介绍，一般在README.md文件内
    long_description_content_type="text/markdown",
    url="https://github.com/Mon1-innovation/MAICA",    #自己项目地址，比如github的项目地址
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.12',
    install_requires=parse_requirements(),
    entry_points={
        'console_scripts': [
            'maica = maica.maica_starter:full_start',
        ],
    },
    package_data={
        'maica': ['env_example'],
    },
)


import os
import sys
from pathlib import Path
from dotenv import load_dotenv
try:
    from maica import maica_ws
except Exception:
    current_dir = Path(__file__).parent
    parent_dir = current_dir.parent
    if str(parent_dir) not in sys.path:
        sys.path.insert(0, str(parent_dir))

import asyncio
import argparse
import colorama

from maica import maica_ws, maica_http, common_schedule
from maica.maica_utils import *
from maica.initializer import *

from maica.maica_http import pkg_init_maica_http
from maica.maica_nows import pkg_init_maica_nows
from maica.maica_utils import pkg_init_maica_utils
def pkg_init_maica():
    pkg_init_maica_http()
    pkg_init_maica_nows()
    pkg_init_maica_utils()

colorama.init(autoreset=True)
initialized = False
_silent = False

def printer(*args, **kwargs):
    global _silent
    if not _silent:
        sync_messenger(*args, **kwargs)

def check_params(envdir: str=None, extra_envdir: list=None, silent=False, **kwargs):
    """This will only run once. Recalling will not take effect, except passing in extra kwargs."""
    global initialized, _silent
    _silent = silent

    def init_parser():
        parser = argparse.ArgumentParser(description="Start MAICA Illuminator deployment")
        parser.add_argument('-e', '--envdir', help='Include external env file for running deployment, specify every time')
        parser.add_argument('-t', '--templates', choices=['print', 'create'], nargs='?', const='print', help='Print config templates or create them in current directory')
        return parser
    
    parser = init_parser()
    args = parser.parse_args()
    envdir = envdir or args.envdir
    templates = args.templates

    def dest_env(envdir: str=None, extra_envdir: list=None):
        if not envdir:
            realpath = get_inner_path('.env')
            printer(info=f'No env file designated, defaulting to {realpath}...', type=MsgType.DEBUG)
            envdir = realpath

        realpath = os.path.abspath(envdir)
        if os.path.isfile(realpath):
            printer(info=f'Loading env file {realpath}...', type=MsgType.DEBUG)
            load_dotenv(dotenv_path=realpath)
        else:
            printer(info=f'{realpath} is not a file, skipping real env file...', type=MsgType.WARN)

        if extra_envdir:
            for edir in extra_envdir:
                realpath = os.path.abspath(edir)
                if os.path.isfile(realpath):
                    printer(info=f'Loading extra env file {realpath}...', type=MsgType.DEBUG)
                    load_dotenv(dotenv_path=realpath)

        realpath = get_inner_path('env_example')
        printer(info=f'Loading env example {realpath} to guarantee basic functions...', type=MsgType.DEBUG)
        if os.path.isfile(realpath):
            load_dotenv(dotenv_path=realpath)
        else:
            raise Exception('env template lost')

    def get_templates():
        with open(get_inner_path('env_example'), 'r', encoding='utf-8') as env_e:
            env_c = env_e.read()
        return env_c
    
    def separate_line(title: str):
        try:
            terminal_width = os.get_terminal_size().columns
        except:
            terminal_width = 40
        line = title.center(terminal_width, '/')
        return line
    
    def print_templates():
        env_c = get_templates()
        print(colorama.Fore.BLUE + separate_line('Begin .env template'))
        print(env_c)
        print(colorama.Fore.BLUE + separate_line('End .env template'))
        exit(0)

    def create_templates():
        env_c = get_templates()
        env_p = os.path.abspath('./.env')
        if os.path.exists(env_p):
            printer(info=f'Config {env_p} already exists, skipping creation...', type=MsgType.WARN)
        else:
            with open(env_p, 'w', encoding='utf-8') as env_f:
                printer(info=f'Generating {env_p}...', type=MsgType.DEBUG)
                env_f.write(env_c)

        printer(info='Creation succeeded, edit them yourself and then start with "maica -c .env"', type=MsgType.INFO)
        exit(0)

    if kwargs:
        for k, v in kwargs.items():
            os.environ[k] = v
        printer(info=f'Added {len(kwargs)} vars to environ.', type=MsgType.DEBUG)

    if not initialized:
        try:
            if templates:
                print_templates() if templates == 'print' else create_templates()
            dest_env(envdir, extra_envdir)
            pkg_init_maica()
            initialized = True
        except Exception as e:
            printer(info=f'Error: {str(e)}, quitting...', type=MsgType.ERROR)
            exit(1)

def check_env_init():
    """We run this only if called to serve. No env is basically not a problem for working as module."""
    if load_env('MAICA_IS_REAL_ENV') == '1':
        return
    else:
        print('''No real env detected, is this workflow?
If it is, at least the imports and grammar are good if you see this.
If not:
    If you're running MAICA for deployment, pass in "--envdir path/to/.env".
    If you're developing with MAICA as dependency, call maica.init() after import.
    Or, you can manually set the necessary env vars.
Quitting...'''
              )
        quit(0)

def check_data_init():
    if not check_marking():
        generate_rsa_keys()
        asyncio.run(create_tables())
        create_marking()
        printer(info="MAICA Illuminator initiation finished", type=MsgType.PRIM_SYS)
    else:
        printer(info="Initiated marking detected, skipping initiation", type=MsgType.DEBUG)

def check_warns():
    if load_env('MAICA_DB_ADDR') == 'sqlite':
        printer(info='\nMAICA using SQLite database detected\nWhile MAICA Illuminator is fully compatible with SQLite, it can be a significant drawback of performance under heavy workload\nIf this is a public service instance, it\'s strongly recommended to use MySQL/MariaDB instead', type=MsgType.DEBUG)
    if not load_env('MAICA_PROXY_ADDR'):
        printer(info='\nInternet proxy absence detected\nMAICA Illuminator needs to access Google and Wikipedia for full functionalities, so you\'ll possibly need a proxy for those\nYou can ignore this message safely if you have direct access to those or have a global proxy set already', type=MsgType.DEBUG)
    if load_env('MAICA_DEV_STATUS') != 'serving':
        printer(info='\nServer status not serving detected\nWith this announcement taking effect, standard clients will stop further communications with MAICA Illuminator respectively\nYou should set DEV_STATUS to \'serving\' whenever the instance is ready to serve', type=MsgType.DEBUG)
    if (load_env('MAICA_MCORE_NODE') and load_env('MAICA_MCORE_USER') and load_env('MAICA_MCORE_PWD')) or (load_env('MAICA_MFOCUS_NODE') and load_env('MAICA_MFOCUS_USER') and load_env('MAICA_MFOCUS_PWD')):
        printer(info='\nOne or more NVwatch nodes detected\nNVwatch of MAICA Illuminator will try to collect nvidia-smi outputs through SSH, which can fail the process if SSH not avaliable\nIf SSH of nodes are not accessable or not wanted to be used, delete X_NODE, X_USER, X_PWD accordingly', type=MsgType.DEBUG)

async def start_all():

    auth_pool, maica_pool = await asyncio.gather(ConnUtils.auth_pool(), ConnUtils.maica_pool())
    kwargs = {"auth_pool": auth_pool, "maica_pool": maica_pool}
    task_ws = asyncio.create_task(maica_ws.prepare_thread(**kwargs))
    task_http = asyncio.create_task(maica_http.prepare_thread(**kwargs))
    task_schedule = asyncio.create_task(common_schedule.schedule_rotate_cache(**kwargs))

    res = await asyncio.wait([
        task_ws,
        task_http,
        task_schedule,
    ], return_when=asyncio.FIRST_COMPLETED)
    await messenger(info="First quit collected, quitting other tasks...", type=MsgType.DEBUG)
    for pending in res[1]:
        pending.cancel()
        await pending
    await messenger(info="All quits collected, doing final cleanup...", type=MsgType.DEBUG)
    await asyncio.gather(auth_pool.close(), maica_pool.close(), return_exceptions=True)
    await messenger(info="Everything done, bye", type=MsgType.DEBUG)
    quit()

def full_start():
    check_params()
    check_env_init()
    check_data_init()
    check_warns()
    asyncio.run(start_all())

if __name__ == "__main__":
    full_start()


import traceback
from typing import *
from maica.maica_utils import *

STRUCTURE_NOT_INTACT = MaicaInputWarning(f'Trigger structure not intact', '400', 'maica_mtrigger_item_bad')

class CommonTrigger():
    template = None
    """Should always exist."""
    name = None
    """Should always exist."""
    def __init__(self, **kwargs):
        try:
            self.name = kwargs.get('name')
            assert self.name, 'Trigger name absent'
        except Exception as e:
            raise e

class _exprop():
    item_list: Optional[list] = None
    curr_item: Optional[str] = None
    suggestion: Optional[bool] = None
    value_limits: Optional[list] = None
    curr_value: Optional[str] = None
    def __init__(self, **kwargs):
        prop_list = ['item_list', 'curr_item', 'suggestion', 'value_limits', 'curr_value']
        try:
            self.item_name = _item_name(**kwargs.get('item_name'))
            """Should always exist."""
            for acceptable_key in prop_list:
                if kwargs.get(acceptable_key) != None:
                    setattr(self, acceptable_key, kwargs.get(acceptable_key))
            if self.item_list:
                assert isinstance(self.item_list, list), 'Item list not list'
                self.item_list = limit_length(self.item_list, 72)
            if self.value_limits:
                assert isinstance(self.value_limits, list), 'Value limits not list'
                assert len(self.value_limits) == 2, 'Value limits not two'
        except Exception as e:
            raise e

class _item_name():
    def __init__(self, **kwargs):
        try:
            self.zh, self.en = kwargs.get('zh'), kwargs.get('en')
            assert self.zh and self.en, 'Item name absent'
        except Exception as e:
            raise e

class CommonAffectionTrigger(CommonTrigger):
    template = 'common_affection_template'
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        
class CommonSwitchTrigger(CommonTrigger):
    template = 'common_switch_template'
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        try:
            self.exprop = _exprop(**kwargs.get('exprop'))
            assert self.exprop.item_list, 'No item list provided for switch'
        except Exception as e:
            raise MaicaInputWarning(f'Trigger {self.name} not intact: {str(e)}')
        
class CommonMeterTrigger(CommonTrigger):
    template = 'common_meter_template'
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        try:
            self.exprop = _exprop(**kwargs.get('exprop'))
            assert self.exprop.value_limits
        except Exception as e:
            raise MaicaInputWarning(f'Trigger {self.name} not intact: {str(e)}')

class CustomizedTrigger(CommonTrigger):
    template = 'customized'
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        try:
            self.exprop = _exprop(**kwargs.get('exprop'))
        except Exception as e:
            raise MaicaInputWarning(f'Trigger {self.name} not intact: {str(e)}')